#ifndef __ACCESS_H__
#define __ACCESS_H__

#include "defines.h"

#define ACCESS_FLAGS_CHECKEXISTS	0x001
#define ACCESS_FLAGS_BESTMATCH		0x002

typedef struct _access			ACCESS;

typedef struct _confhostipentry	CONFHOSTIPENTRY;
typedef struct _confhost		CONFHOST;
typedef struct _confip			CONFIP;
typedef struct _confptr			CONFPTR;
typedef struct _confentry		CONFENTRY;

struct _confhostipentry
{
	int				expires;

	char			*pattern;
	char			*password;
	char			*username;
	char			*hostname;
	ircd_uint32_t		ip_net;
	ircd_uint32_t		ip_mask;
	char			*ipstring;
#define ALLOW_FLAGS_CHECKHOSTNAME	0x001
	u_int			flags;
	char			*classname;
	void			*classptr;
	char			*data;
	char			*comment;

	CONFHOSTIPENTRY		*ch_prev;
	CONFHOSTIPENTRY		*ch_next;
};

struct _confptr
{
	CONFHOSTIPENTRY	*entry;
	CONFENTRY		*sub;
	CONFPTR			*next;
};

struct _confentry
{
	CONFPTR			*table;
	int				num;
	int				size;
};

struct _confip
{
	CONFENTRY		sort_table[256][33];
#if 0
	CONFENTRY		lin_list;
#endif
};

struct _confhost
{
	CONFENTRY		rev_table;
	CONFENTRY		for_table;
	CONFENTRY		lin_list;	/* step through with ->next */
};

struct _access
{
	CONFHOST		*conf_host;
	CONFIP			*conf_ip;
	CONFHOSTIPENTRY		*conf_all; /* ->ch_next */
};

int access_init(ACCESS *access);
int access_match_hostname(ACCESS *access, char *username, char *hostname, u_int flags, CONFHOSTIPENTRY **entryptr);
int access_match_ipaddr(ACCESS *access, char *username, char *ipstring, ircd_uint32_t ipaddr, u_int flags, CONFHOSTIPENTRY **entryptr);
int access_match_ipaddr_default(ACCESS *access, char *username, char *ipstring, ircd_uint32_t ipaddr, u_int flags, CONFHOSTIPENTRY **entryptr);
int access_add(ACCESS *access, char *username, char *hostname, char *ipstring, char *class, char *data, int expires, u_int flags, int aflags, CONFHOSTIPENTRY **existptr, char *comment);
int access_clear(ACCESS *access, u_int flags);
int access_flush(ACCESS *access, u_int flags);
int access_del(ACCESS *access, CONFHOSTIPENTRY *chentry, u_int flags);

#endif
