#ifndef __IRCDRES_H__
#define __IRCDRES_H__

#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>


typedef struct _ircdres			IRCDRES;
typedef struct _ircdres_result	IRCDRES_RESULT;
typedef struct _ircdres_request	IRCDRES_REQUEST;
typedef struct _ircdres_hostent	IRCDRES_HOSTENT;

typedef int (*ircdres_func_t)(IRCDRES *, IRCDRES_RESULT *, void *);

struct _ircdres
{
	int				res_fd;
	struct sockaddr_in	*res_ns_ipaddrs;
	u_int			res_ns_num;
	u_int			res_ns_curnum;
};

struct _ircdres_hostent
{
	struct hostent	he_he;
	char			*he_hbuf;
};

struct _ircdres_result
{
	u_short			res_type;
	u_int			res_ttl;
	IRCDRES_HOSTENT	*res_he;
};

struct _ircdres_request
{
	u_int			req_qtype;

#define REQ_FLAGS_CLOSED	0x001
	u_int			req_flags;

	ircdres_func_t	req_callback;
	void			*req_callback_arg;

	char			*req_name;
	struct in_addr	req_inaddr;

	int				req_sends;

	u_short			req_id;

	IRCDRES			*req_res;

	IRCDRES_REQUEST	*req_prev;
	IRCDRES_REQUEST	*req_next;
	IRCDRES_REQUEST	*req_h_prev;
	IRCDRES_REQUEST	*req_h_next;
};

int ircdres_deinit(IRCDRES *resptr);
int ircdres_init(IRCDRES **rptr);
int ircdres_set_ns(IRCDRES *resptr, struct sockaddr_in *nameservers, int num);
int ircdres_request_del(IRCDRES_REQUEST *request, int call_callback);
int ircdres_gethostbyname(IRCDRES *resptr, char *name, ircdres_func_t callback, void *callback_arg, IRCDRES_HOSTENT **result, IRCDRES_REQUEST **request);
int ircdres_gethostbyaddr(IRCDRES *resptr, struct in_addr *addr, ircdres_func_t callback, void *callback_arg, IRCDRES_HOSTENT **result, IRCDRES_REQUEST **request);
int ircdres_get_answer(IRCDRES *resptr);
int ircdres_deinit_reload(IRCDRES_REQUEST **reqptr);
int ircdres_init_reload(IRCDRES_REQUEST *reqptr);
int ircdres_requests_outstanding(void);
int ircdres_request_resend(IRCDRES_REQUEST *reqptr);

#endif
