#ifndef __STRUCTS_H__
#define __STRUCTS_H__

#include "defines.h"
#include "fdlist.h"
#include "queue.h"
#include "res.h"

#define ALLOW_FLAGS_CHECKHOSTNAME	0x001

typedef struct _ircd_conn			IRCD_CONN;
typedef struct _ircd_auth			IRCD_AUTH;
typedef struct _ircd_rejecthold		IRCD_REJECTHOLD;
typedef struct _ircd_listener		IRCD_LISTENER;
typedef struct _ircd_userflag   	IRCD_USERFLAG;
typedef struct _ircd_nethash		IRCD_NETHASH;
typedef struct _ircd_ipentry		IRCD_IPENTRY;
typedef struct _ircd_stringlist		IRCD_STRINGLIST;
typedef struct _ircd_classentry		IRCD_CLASSENTRY;
typedef struct _ircd_client			IRCD_CLIENT;
typedef struct _ircd_whowas			IRCD_WHOWAS;
typedef struct _ircd_ban			IRCD_BAN;
typedef struct _ircd_channel		IRCD_CHANNEL;
typedef struct _ircd_channellist	IRCD_CHANNELLIST;
typedef struct _ircd_clientlist		IRCD_CLIENTLIST;
typedef struct _globalstats			GLOBALSTATS;
typedef struct _ircd_invite			IRCD_INVITE;
typedef struct _ircdres_reqlist		IRCDRES_REQLIST;

struct _ircd_conn
{
	FDLISTINFO		fdlistinfo;
#define IRCD_CONN_TYPE_CLIENT		0x001
#define IRCD_CONN_TYPE_AUTH			0x002
#define IRCD_CONN_TYPE_LISTENER		0x004
#define IRCD_CONN_TYPE_RES			0x008
#define IRCD_CONN_TYPE_REJECTHOLD	0x010
#define IsRes(__cptr)			((__cptr)->conn_type == IRCD_CONN_TYPE_RES)
#define IsAuth(__cptr)			((__cptr)->conn_type == IRCD_CONN_TYPE_AUTH)
#define IsListener(__cptr)		((__cptr)->conn_type == IRCD_CONN_TYPE_LISTENER)
#define IsRejecthold(__cptr)	((__cptr)->conn_type == IRCD_CONN_TYPE_REJECTHOLD)
#define IsCClient(__cptr)		((__cptr)->conn_type == IRCD_CONN_TYPE_CLIENT)
	int				conn_type;
	u_int			conn_lasttime;
	void			*conn_ptr;
};

struct _ircd_auth
{
	IRCD_CONN		*a_connptr;
	IRCD_CLIENT		*a_clptr;
	IQUEUE			a_queue_in;
	u_int			a_firsttime;

	IRCD_AUTH		*a_prev;
	IRCD_AUTH		*a_next;
};

struct _ircd_rejecthold
{
	IRCD_CONN		*rh_connptr;

	u_int			rh_firsttime;
	u_int			rh_lasttime;

	u_int			rh_rem_ipaddr;
	u_short			rh_rem_port;
	u_int			rh_loc_ipaddr;
	u_short			rh_loc_port;


	IRCD_REJECTHOLD	*rh_next;
	IRCD_REJECTHOLD	*rh_prev;
};

struct _ircd_listener
{
	IRCD_CONN		*l_connptr;

	u_int			l_ipaddr;
	u_short			l_port;
#define IRCD_LISTENER_FLAGS_ILLEGAL	0x001
	u_int			l_flags;

	u_int			l_num_on;

	u_int			l_lasttime;
	u_int			l_firsttime;
	char			l_ipstring[16];

	IRCD_CLIENT		*l_clients;

	IRCD_LISTENER	*l_prev;
	IRCD_LISTENER	*l_next;
};

struct _ircd_userflag
{
	int				flag;
	char			*flagstr;
	int				opersonly;
};

struct _ircd_ipentry
{
	void			*extra_ptr;
	ircd_uint32_t	ipaddr;

	int				local_connect_num;
	int				local_connect_lasttime;
	int				remote_connect_num;
	int				remote_connect_lasttime;

	IRCD_NETHASH	*nhptr;
	IRCD_CLIENT		*clients;

	IRCD_IPENTRY	*prev;
	IRCD_IPENTRY	*next;
};

struct _ircd_nethash
{
	void			*extra_ptr;
	ircd_uint32_t	ipaddr;

	int				accept_rate_num;
	int				accept_rate_time;
	int				accept_rate_persec;

	int				accept_rate_maxtime; /* used for removing logic */
	int				client_rate_maxtime; /* used for removing logic */

	int				accept_throttle_notice;

	int				client_rate_num;
	int				client_rate_time;

	int				flags;
#define IRCD_NETHASH_FLAGS_ATHROTTLED	0x001
#define IRCD_NETHASH_FLAGS_CTHROTTLED	0x002

	int				local_connect_num;
	int				local_connect_lasttime;
	int				remote_connect_num;
	int				remote_connect_lasttime;

	IRCD_IPENTRY	*ipaddrs;
	IRCD_CLIENT		*clients;

	IRCD_NETHASH	*c_prev;
	IRCD_NETHASH	*c_next;
	IRCD_NETHASH	*h_prev;
	IRCD_NETHASH	*h_next;
	IRCD_NETHASH	*t_prev;
	IRCD_NETHASH	*t_next;
};

struct _ircd_stringlist
{
	char				*string;
	IRCD_STRINGLIST		*next;
	IRCD_STRINGLIST		*prev;
};

struct _ircd_classentry
{
	void		*extra_ptr;
	char		*classname;
	int			maxsendq;
	int			pingfreq;
	int			connfreq;
	int			maxclients;
	int			maxchannels;
	int			uhlimit;
	int			hlimit;
	int			uvlimit;
	int			vlimit;
	int			floodnum;
	int			floodtime;
	int			floodexit;
#define CLASS_FLAGS_ILLEGAL				0x001
#define CLASS_FLAGS_BYPASSLIMITS		0x002
#define CLASS_FLAGS_BYPASSDENY			0x004
#define CLASS_FLAGS_NOTILDE				0x008
#define CLASS_FLAGS_IDENTONLY			0x010
#define CLASS_FLAGS_HIDEHOSTNAME		0x020
#define CLASS_FLAGS_BYPASSNICKNAMEDENY	0x040
#define CLASS_FLAGS_BYPASSIRCNAMEDENY	0x080
#define CLASS_FLAGS_BYPASSCHANNELDENY	0x100
	char		hostname[80];
	int			flags;
	int			num_on;
	int			ipaddr_rate_time;
	int			ipaddr_rate_num;
	u_int		lastconnect;
	IRCD_CLIENT	*clients;
};

struct _globalstats
{
	int		m_invis;
	int		g_invis;
	int		m_opers;
	int		g_opers;
	int		m_clients;
	int		g_clients;
	int		m_servers;
	int		g_servers;
	int		g_maxclients;
	int		g_maxclientstime;
	int		g_maxservers;
	int		g_maxserverstime;
	int		m_maxconn;
	int		m_maxconntime;
	int		m_maxclients;
	int		m_maxclientstime;
	int		m_maxservers;
	int		m_maxserverstime;
	int		m_totclients;
	int		m_totclientstime;
	int		m_unknown;
	int		g_serversbursting;
	int		channels;
	int		maxchannels;
	int		maxchannelstime;

	int		bw_last_check;
	int		bw_c_incoming_bytes;
	int		bw_c_outgoing_bytes;
	int		bw_s_incoming_bytes;
	int		bw_s_outgoing_bytes;
	int		bw_c_incoming_rate;	/* last 5 sec rate...in bytes/s  */
	int		bw_c_outgoing_rate;	/* last 5 sec rate...in bytes/s  */
	int		bw_s_incoming_rate;	/* last 5 sec rate...in bytes/s  */
	int		bw_s_outgoing_rate;	/* last 5 sec rate...in bytes/s  */
};

struct _ircdres_reqlist
{
	IRCDRES_REQUEST	*rl_req;
	u_int			rl_firsttime;
	u_int			rl_lasttime;

	IRCDRES_HOSTENT	*rl_reverse;
	IRCDRES_HOSTENT	*rl_forward;

	IRCDRES_REQLIST *rl_prev;
	IRCDRES_REQLIST *rl_next;
};

struct _ircd_client
{
	IRCD_CONN		*c_connptr;

	void			*c_extra_ptr;

	IRCD_CLIENT		*c_h_prev;
	IRCD_CLIENT		*c_h_next;

	IRCD_IPENTRY	*c_ipaddr_ptr;
	IRCD_CLIENT		*c_i_prev;
	IRCD_CLIENT		*c_i_next;
	IRCD_CLIENT		*c_n_prev;
	IRCD_CLIENT		*c_n_next;

	IRCD_CLIENT		**c_oslist;
	IRCD_CLIENT		*c_os_prev;
	IRCD_CLIENT		*c_os_next;

	IRCD_CLIENT		*c_from;

	IRCD_CLIENT		*c_servptr;		/* the server to which the client belongs*/
	IRCD_CLIENT		*c_users;			/* for servers, users on the server */
	IRCD_CLIENT		*c_servers;		/* for servers, servers on this server */

	IRCD_CLIENT		*c_u_prev;
	IRCD_CLIENT		*c_u_next;

	IRCD_INVITE		*c_invites_list;
	int				c_invites_num;

	unsigned int	c_hashv;

	int				c_hopcount;
	char			c_nicksent;

	int				c_creation;

	int				c_firsttime;
	int				c_lasttime;
	int				c_lastparse;

	char			name[HOSTLEN+1];
	char			username[USERLEN+1];
	char			hostname[HOSTLEN+1];
	char			servername[HOSTLEN+1];
	char			ircname[REALLEN+1];

	char			*c_away;

	IRCD_CHANNELLIST	*c_channels;
	int					c_numchannels;

#define CLIENT_TYPE_UNKNOWN			0x000
#define CLIENT_TYPE_CLIENT			0x001
#define CLIENT_TYPE_SERVER			0x002
#define IsUnknown(__cptr)		((__cptr)->c_type == CLIENT_TYPE_UNKNOWN)
#define IsServer(__cptr)		((__cptr)->c_type == CLIENT_TYPE_SERVER)
#define IsClient(__cptr)		((__cptr)->c_type == CLIENT_TYPE_CLIENT)
#define IsPerson(__cptr)		(IsClient(__cptr) && IsReg(__cptr))
#define MyClient(__cptr)		(IsLocal(__cptr) && IsClient(__cptr))
	int				c_type;

#define CLIENT_FLAGS_DOINGAUTH		0x0001
#define CLIENT_FLAGS_DOINGDNS		0x0002
#define CLIENT_FLAGS_DOINGCONNECT	0x0004
#define CLIENT_FLAGS_ILLEGAL		0x0008
#define CLIENT_FLAGS_LOCAL			0x0010
#define CLIENT_FLAGS_DEAD			0x0020
#define CLIENT_FLAGS_REG			0x0040
#define CLIENT_FLAGS_HOSTMASK		0x0080	/* *name == '*' */
#define CLIENT_FLAGS_GOTUSER		0x0100
#define CLIENT_FLAGS_GOTAUTH		0x0200
#define CLIENT_FLAGS_DOESTS			0x0400
#define CLIENT_FLAGS_KILLED			0x0800
#define CLIENT_FLAGS_PINGSENT		0x1000
#define CLIENT_FLAGS_SCONNECT		0x2000
#define CLIENT_FLAGS_NOSENDQUIT		0x4000
#define CLIENT_FLAGS_SENTEOB		0x8000
#define CLIENT_FLAGS_GOTEOB			0x10000
#define IsLocal(__cptr)				((__cptr)->c_flags & CLIENT_FLAGS_LOCAL)
#define IsDoingAuth(__cptr)			((__cptr)->c_flags & CLIENT_FLAGS_DOINGAUTH)
#define IsDoingDns(__cptr)			((__cptr)->c_flags & CLIENT_FLAGS_DOINGDNS)
#define IsIllegal(__cptr)			((__cptr)->c_flags & CLIENT_FLAGS_ILLEGAL)
#define IsDoingConnect(__cptr)		((__cptr)->c_flags & CLIENT_FLAGS_DOINGCONNECT)
#define IsDead(__cptr)				((__cptr)->c_flags & CLIENT_FLAGS_DEAD)
#define IsReg(__cptr)				((__cptr)->c_flags & CLIENT_FLAGS_REG)
#define DoesTS(__cptr)				((__cptr)->c_flags & CLIENT_FLAGS_DOESTS)
#define CliSetSentEOB(__x)          ((__x)->c_flags |= CLIENT_FLAGS_SENTEOB)
#define CliSentEOB(__x)             ((__x)->c_flags & CLIENT_FLAGS_SENTEOB)
#define CliSetGotEOB(__x)           ((__x)->c_flags |= CLIENT_FLAGS_GOTEOB)
#define CliGotEOB(__x)              ((__x)->c_flags & CLIENT_FLAGS_GOTEOB)

	int				c_flags;

	int				c_cflags;

#define CLIENT_UMODES_OPER			0x00001
#define CLIENT_UMODES_LOCOP			0x00002
#define CLIENT_UMODES_BMODE			0x00004
#define CLIENT_UMODES_CMODE			0x00008
#define CLIENT_UMODES_DMODE			0x00010
#define CLIENT_UMODES_FMODE			0x00020
#define CLIENT_UMODES_GMODE			0x00040
#define CLIENT_UMODES_IMODE			0x00080
#define CLIENT_UMODES_KMODE			0x00100
#define CLIENT_UMODES_LMODE			0x00200
#define CLIENT_UMODES_NMODE			0x00400
#define CLIENT_UMODES_SMODE			0x00800
#define CLIENT_UMODES_UMODE			0x01000
#define CLIENT_UMODES_WMODE			0x02000
#define CLIENT_UMODES_ZMODE			0x04000
#define CLIENT_UMODES_YMODE			0x08000

#define SEND_UMODES	(CLIENT_UMODES_OPER|CLIENT_UMODES_WMODE|CLIENT_UMODES_IMODE)

#define IsAnOper(__x)				((__x)->c_umodes & (CLIENT_UMODES_OPER | \
												CLIENT_UMODES_LOCOP))
#define IsOper(__x)					((__x)->c_umodes & CLIENT_UMODES_OPER)
#define IsLocOp(__x)				((__x)->c_umodes & CLIENT_UMODES_LOCOP)
#define IsInvisible(__x)			((__x)->c_umodes & CLIENT_UMODES_IMODE)
	int				c_umodes;

	IRCD_WHOWAS		*c_whowas;

	ircd_uint32_t	c_rem_ipaddr;
	ircd_uint16_t	c_rem_port;


	/* END OF REMOTE-ONLY CLIENT */

	IRCD_AUTH		*c_auth_ptr;

	int				c_oline_flags;

	int				c_idletime;

#define CLIENT_NFLAGS_OWALLOPS			0x0001
#define CLIENT_NFLAGS_SWALLOPS			0x0002
#define CLIENT_NFLAGS_STATSNOTICES		0x0004
#define CLIENT_NFLAGS_OKILLS			0x0008
#define CLIENT_NFLAGS_SKILLS			0x0010
#define CLIENT_NFLAGS_SNOTICES			0x0020
#define CLIENT_NFLAGS_CLICONNECTS		0x0040
#define CLIENT_NFLAGS_CLIDISCONNECTS	0x0080
#define CLIENT_NFLAGS_THROTTLES			0x0100
#define CLIENT_NFLAGS_NICKCHANGES		0x0200
#define CLIENT_NFLAGS_IPMISMATCHES		0x0400
#define CLIENT_NFLAGS_LWALLOPS			0x0800
#define CLIENT_NFLAGS_CONNECTS			0x1000
#define CLIENT_NFLAGS_SQUITS			0x2000
	int				c_nflags;

	char			c_ipstring[16];

	char			c_passwd[PASSWDLEN+1];

	int				c_hostmask;
	char			*c_connectby;

	int				c_pingfreq;
	int				c_maxsendq;
	int				c_lastqsize;

	int				c_send_msg;
	int				c_send_bytes;
	int				c_send_kbytes;
	int				c_recv_msg;
	int				c_recv_bytes;
	int				c_recv_kbytes;

	int				c_writetries;

	int				c_rate_msgs;
	int				c_rate_time;
	int				c_rate_exit_num;
	int				c_rate_lastflood;
	int				c_rate_msg_limit;
	int				c_rate_time_limit;
	int				c_rate_exit_num_limit;

	IRCD_CLIENT		**c_flood_list;
	IRCD_CLIENT		*c_f_prev;	/* Flooders list */
	IRCD_CLIENT		*c_f_next;	/* Flooders list */

	char			c_authusername[30];

	IQUEUE			c_queue_in;
	IQUEUE			c_queue_out;

	IRCDRES_REQLIST	*c_res_req;

	ircd_uint32_t	c_loc_ipaddr;
	ircd_uint16_t	c_loc_port;

	IRCD_LISTENER	*c_listener; /* ptr to the listener (IRCD_CLIENT *) */

	IRCD_CLIENT		*c_l_prev;    /* Listeners list */
	IRCD_CLIENT		*c_l_next;    /* Listeners list */

	IRCD_CLIENT		*c_lo_prev;	/* Locals list */
	IRCD_CLIENT		*c_lo_next;	/* Locals list */

	IRCD_CLASSENTRY		*c_class;
	IRCD_CLIENT		*c_cl_prev;	/* Class list */
	IRCD_CLIENT		*c_cl_next;	/* Class list */


	void			*c_dns_req;

#define IRCD_CAPAB_VALUE_IPADDR			0x001
#define IRCD_CAPAB_VALUE_ZIP			0x002
#define IRCD_CAPAB_VALUE_NOQUITSTORM	0x004
#define IRCD_CAPAB_VALUE_EMODE			0x008

#define IsCapable(__x, __y)	((__x)->c_capab_flags & (__y))
	int				c_capab_flags;

};

struct _ircd_whowas
{
	void			*extra_ptr;
	unsigned int	hashv;
	char			*name;
	char			*username;
	char			*hostname;         
	char			*servername;       
	char			*realname;
	int				logoff;
	IRCD_CLIENT		*online; /* Pointer to new nickname for chasing or NULL */
	IRCD_WHOWAS		*next;  /* for hash table... */
	IRCD_WHOWAS		*prev;  /* for hash table... */
	IRCD_WHOWAS		*cnext; /* for client struct linked list */
	IRCD_WHOWAS		*cprev; /* for client struct linked list */
};

struct _ircd_ban
{
	void		*extra_ptr;
	char		*banstring;
	char		*who;
	int			bantime;

	char			*firstpart;
	ircd_uint32_t	net;
	ircd_uint32_t	mask;


	IRCD_BAN	*next;
	IRCD_BAN	*prev;
};

struct _ircd_channel
{
	void			*extra_ptr;
	unsigned int	hashv;

	int				creation;

#define CHANNEL_MODES_SECRET		0x001
#define CHANNEL_MODES_PRIVATE		0x002
#define CHANNEL_MODES_MODERATED		0x004
#define CHANNEL_MODES_TOPICLIMIT	0x010
#define CHANNEL_MODES_INVITEONLY	0x020
#define CHANNEL_MODES_NOPRIVMSGS	0x040

/* for clientlist->modes */
#define CHANNEL_MODES_CHANOP		0x001
#define CHANNEL_MODES_VOICE			0x002
#define CHANNEL_MODES_DEOPPED		0x004
	int				modes;
	char			*key;
	int				limit;

	char			*topic;
	char			*topic_who;
	int				topic_time;

	IRCD_CLIENTLIST	*members;
	int				numusers;

	int				nbans;
	IRCD_BAN		*bans;

	IRCD_CHANNEL	*prev;
	IRCD_CHANNEL	*next;

	IRCD_CHANNEL	*h_prev;
	IRCD_CHANNEL	*h_next;


	char			name[1];
};

struct _ircd_channellist
{
	IRCD_CHANNEL		*chptr;
	int					modes;
	IRCD_CLIENTLIST		*clptr;
	IRCD_CHANNELLIST	*prev;
	IRCD_CHANNELLIST	*next;
};

struct _ircd_clientlist
{
	IRCD_CLIENT			*cptr;
	int					modes;
	IRCD_CHANNELLIST	*chlptr;
	IRCD_CLIENTLIST		*prev;
	IRCD_CLIENTLIST		*next;
};

struct _ircd_invite
{
	char			*channel;
	IRCD_INVITE		*prev;
	IRCD_INVITE		*next;
};

#endif


