/*
**   IRC - Internet Relay Chat, source/ircd.c
**   Copyright (C) 1999 Chris Behrens
**
**   Portions Copyright (C) 1990 Jarkko Oikarinen and
**                      University of Oulu, Computing Center
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ircd_includes.h"
#include "commands.h"
#include <arpa/nameser.h>

RCSID("$Id: ircd.c,v 1.19 2001/09/30 11:02:14 cbehrens Exp $");


typedef struct _ircd_data	IRCD_DATA;

struct _ircd_data
{
	char			*Conffile;

	/* LEAVE THE ABOVE 3 FIRST AND NEVER CHANGE !!! */

	int				ircd_pagesize;
	int				Fdlist_num;
	FDLIST			*Fdlist_main;
	FDLIST			*Fdlist_clients;
	int				*Fdlist_lasttime;
	IRCD_CONN		**Fds;				/* for easy fd -> client lookup */
	int				Fds_size;
	IRCD_CLIENT		*Locals;
	IRCD_AUTH		*Auths;
	IRCD_CLIENT		**Clients;
	IRCD_CLIENT		**Opers;
	IRCD_CLIENT		**Servers;
	IRCD_CLIENT		**Flooders;
	IRCD_CHANNEL	*Channels;
	GLOBALSTATS		*Globalinfo;
	IRCD_REJECTHOLD	*Rejectholds;
	IRCD_MOTD		*Motd;
	int				Sendbuffersize;
	int				Recvbuffersize;

	unsigned int	*Sentalong;
	unsigned int	Sentalong_marker;
	unsigned int	*Sentalong_end;
	int				Sentalong_size;

	IRCD_CONN		*Ircd_resconn;
	IRCDRES_REQUEST	*ResRequests;
	IRCDRES_REQLIST	*ResReqlist;

	int				Ircd_zero_fd;

	IRCDCONF		Ircd_conf;

	IRCD_CLIENT		*meptr;

	int				queue_size;
	int				queue_malloced_size;

	void			*res_requests;
	void			*res_lastrequest;
	char			**NumericArr;
	int				NumericArrSize;

	void			*whowasarr;
	int				whowas_size;
	void			**whowashash;
	int				whowas_hash_size;
	int				whowas_next;
	void			*Vars;
	void			**client_hash;
	int				client_hash_size;
	void			**channel_hash;
	int				channel_hash_size;
	void			***iphash;
	int				iphash_size;
	void			*clones;
	void			*throttles;
};

int				Auths_did_delete = 0;
int				Flooders_did_delete = 0;
int				Fdlist_num = 100;
char			*Conffile;
int				ircd_pagesize;
FDLIST			*Fdlist_main;
static FDLIST	*Fdlist_clients;
static int		*Fdlist_lasttime;
int				Fds_size;
IRCD_CONN		**Fds;				/* for easy fd -> client lookup */
IRCDRES_REQLIST	*Reqlist = NULL;
IRCD_CLIENT		*Locals = NULL;			/* local clients */
IRCD_AUTH		*Auths = NULL;
IRCD_CLIENT		**Clients = NULL;
IRCD_CLIENT		**Opers = NULL;
IRCD_CLIENT		**Servers = NULL;
IRCD_CLIENT		**Flooders = NULL;
IRCD_CHANNEL	*Channels = NULL;
GLOBALSTATS		*Globalinfo;
IRCD_REJECTHOLD	*Rejectholds = NULL;
IRCD_MOTD		*Motd = NULL;
int				Sendbuffersize = 0;
int				Recvbuffersize = 0;
unsigned int	*Sentalong;
unsigned int	Sentalong_marker;
unsigned int	*Sentalong_end;
int				Sentalong_size;
IRCD_CLIENT		*meptr;
int				Ircd_zero_fd = -1;
IRCDCONF		_ircd_conf;
IRCD_CONN		*Ircd_resconn;

int				Now;
int				DoRehash = 0;
char			*RehashWho = NULL;
int				Rehashed = 0;
int				Autoconncheck = 0;
int				DoReload = 0;


static char		*ircdcore_versionstring;
static int		*ircdcore_data_format_version;
static char		**ircdcore_data;
static int		*ircdcore_data_size;
static FDLIST_POLLFD *_pfdarray = NULL;
static int		_pfdarray_size = 0;

char	_ircdso_versionstring[] = VERSION;
int		_ircdso_version = 2;


static RETSIGTYPE _ircd_sighup_handler(int sig);
static int _auth_process(IRCD_AUTH *aptr);
static int _ircd_pollfds(FDLIST *fdlist, int secs);
static void flood_check(void);
static void dns_check(void);
static void auth_check(void);
static void ping_check(void);
static void kline_check(void);
static void rejecthold_check(void);
static int autoconn_check(void);
static int _ircdso_firsttime(char *conf_file);
static int _ircdso_init(int firsttime);
static int _ircdso_deinit(void);

static RETSIGTYPE _ircd_sighup_handler(int sig)
{
	DoRehash = 1;
	if (!RehashWho)
		RehashWho = MyStrdup("SIGHUP");
	signal(sig, _ircd_sighup_handler);
}

static RETSIGTYPE _ircd_sigusr1_handler(int sig)
{
	DoReload = 1;
	signal(sig, _ircd_sigusr1_handler);
}

static int _auth_process(IRCD_AUTH *aptr)
{
	char	*buf = NULL;
	int		len = 0;
	short	remp, locp;
	char	ruser[30];
	char	tuser[128];
	char	*blah = tuser;

	if (queue_data_get(&(aptr->a_queue_in), &buf, &len) < 0)
		return 0;
	if (len >= sizeof(tuser))
		blah = (char *)MyMalloc(len+1);
	memcpy(blah, buf, len);
	blah[len] = '\0';

	if ((len > 10) && sscanf(blah, "%hd , %hd : USERID : %*[^:]: %10s",
			&remp, &locp, ruser) == 3)
	{
		register char *s = strrchr(blah, ':') + 1;
		register char *t = ruser;
		register char *ending = ruser + sizeof(ruser) - 1;

		for(;*s && (t < ending);s++)
		{
			if (!isspace(*s) && (*s != ':') && (*s != '@'))
				*t++ = *s;
		}
		*t = '\0';
		support_strncpy(aptr->a_clptr->c_authusername, ruser,
				sizeof(aptr->a_clptr->c_authusername));
		support_strncpy(aptr->a_clptr->username, ruser,
				sizeof(aptr->a_clptr->username));
		if (*ruser)
			aptr->a_clptr->c_flags |= CLIENT_FLAGS_GOTAUTH;
	}
	else
	{
		register char *ptr = blah;

		while(*ptr && (*ptr != '\n') && (*ptr != '\r'))
			ptr++;

		if (!*ptr)
		{
			if (blah != tuser)
				MyFree(blah);
			return -1;
		}
	}
	if (blah != tuser)
		MyFree(blah);
	return 0;
}

static int _ircd_pollfds(FDLIST *fdlist, int secs)
{
	register int	i;
	int				numfds;
	int				ret;
	int				nfds;
	static char		readbuf[1024*64];
	static int		readbufsize = 1024*64;
	IRCD_CONN		*cptr;
	char			exitbuf[512];

	if (!fdlist->pfd_num)
		return 0;

	if (fdlist->pfd_num > _pfdarray_size)
	{
		FDLIST_POLLFD 	*tmp;

		_pfdarray_size = fdlist->pfd_num + 64;
		tmp = (FDLIST_POLLFD *)MyRealloc(_pfdarray,
					_pfdarray_size * sizeof(FDLIST_POLLFD));
		if (!tmp)
			return -ENOMEM;
		_pfdarray = tmp;
	}

	BCOPY(fdlist->pfd_array, _pfdarray, sizeof(FDLIST_POLLFD)*fdlist->pfd_num); 
	numfds = fdlist->pfd_num;

	nfds = poll(_pfdarray, numfds, secs*1000);
	Now = time(NULL);
	if (nfds < 0) /* lets return on EINTR/EAGAIN also */
	{
		return nfds;
	}
	for(i=0;nfds && i<numfds;i++)
	{
		if (!_pfdarray[i].revents)
			continue;
		nfds--;
		if ((cptr = GET_FDS(_pfdarray[i].fd)) == NULL)
			continue;
		cptr->conn_lasttime = Now;
		if (IsRes(cptr))
		{
			(void)ircdres_get_answer((IRCDRES *)cptr->conn_ptr);
			continue;
		}
		if (_pfdarray[i].revents & ~(POLLIN|POLLOUT)) /* some error */
			_pfdarray[i].revents |= POLLIN; /* will close() upon read()*/
		if (_pfdarray[i].revents & POLLIN)
		{
			if (IsListener(cptr))
			{
				IRCD_LISTENER		*lptr = (IRCD_LISTENER *)cptr->conn_ptr;
				struct sockaddr_in	sin;
				int					sinsize = sizeof(sin);
				int 				ipaddr;

				ret = accept(cptr->fdlistinfo.fd,
							(struct sockaddr *)&sin, &sinsize);
				if (ret < 0)
					continue;
				lptr->l_lasttime = Now;
				ipaddr = ntohl(sin.sin_addr.s_addr);
				if (iphash_handle_accept(ipaddr))
				{
					int num;

					if ((num=(int)vars_value_get_bynum(IRCD_VAR_REJECT_HOLD)) > 0)
					{
						if (client_add_rejecthold(ret, &sin) >= 0)
							continue;
					}
					close(ret);
					continue;
				}

				if (!access_match_ipaddr(&(_ircd_conf.deny),
					NULL, NULL, ipaddr, 0, NULL))
				{
					DEBUG(DEBUG_ALL, ("denying accept from %s\n",
						inet_ntoa(sin.sin_addr)));
					close(ret);
				}
				else if (client_add_local(lptr, ret, &sin) < 0)
					close(ret);
				continue;
			}
			ret = read(cptr->fdlistinfo.fd, readbuf, readbufsize-1);
			if ((ret < 0) && 
					((errno == EAGAIN) || (errno == EINTR)
						|| (errno == EWOULDBLOCK)))
				continue;
			if (ret <= 0)
			{
				IRCD_CONN	*cptrkeep;

				if (ret == 0)
					sprintf(exitbuf, "EOF of client");
				else
					sprintf(exitbuf, "Read error %i: %s", errno,
						(errno == 0) ? "Undefined error" : 
								support_strerror(errno));
				if (!IsAuth(cptr))
				{
					conn_del(meptr, cptr, exitbuf);
					continue;
				}
				/* auth is done. */
				cptrkeep = ((IRCD_AUTH *)cptr->conn_ptr)->a_clptr->c_connptr;
				conn_del(meptr, cptr, exitbuf);
				cptr = cptrkeep; /* fall through to the ircd_parse */
				if ((int)vars_value_get_bynum(IRCD_VAR_SHOW_HEADERS))
					send_to_client((IRCD_CLIENT *)cptr->conn_ptr, "NOTICE * :*** Ident request failed.");
			}
			else
				readbuf[ret] = '\0';

			if (IsRejecthold(cptr))
				continue;

			if (ret > 0)
			{
				if (IsAuth(cptr))
				{
					IRCD_AUTH	*aptr = (IRCD_AUTH *)cptr->conn_ptr;
					IRCD_CONN	*cptrkeep;
					int			sh = (int)vars_value_get_bynum(IRCD_VAR_SHOW_HEADERS);

					cptrkeep = aptr->a_clptr->c_connptr;
					if (queue_data_add(&(aptr->a_queue_in), readbuf, ret) >= 0)
					{
						Globalinfo->bw_c_incoming_bytes += ret;

						if (_auth_process(aptr) < 0)
						{
							continue;
						}
						if (sh)
							send_to_client((IRCD_CLIENT *)cptrkeep->conn_ptr, "NOTICE * :*** Ident reply received.");
						conn_del(meptr, cptr, "Auth Done.");
					}
					else
					{
						if (sh)
							send_to_client((IRCD_CLIENT *)cptrkeep->conn_ptr, "NOTICE * :*** Ident request failed: add2queue failed.");
						conn_del(meptr, cptr, "Couldn't add to RecvQ");
					}
					cptr = cptrkeep; /* fall through to the ircd_parse */
					((IRCD_CLIENT *)cptr->conn_ptr)->c_flags &= ~CLIENT_FLAGS_DOINGAUTH;
				}
				else if (IsCClient(cptr))
				{
					IRCD_CLIENT	*clptr = (IRCD_CLIENT *)cptr->conn_ptr;
 
					clptr->c_lasttime = Now;
					if (queue_data_add(&(clptr->c_queue_in), readbuf, ret) < 0)
					{
						conn_del(meptr, cptr, "Couldn't add to RecvQ");
						continue;
					}
					clptr->c_recv_bytes += ret;
					if (clptr->c_recv_bytes >= 1024)
					{
						clptr->c_recv_kbytes += clptr->c_recv_bytes / 1024;
						clptr->c_recv_bytes %= 1024;
					}
					if (IsClient(clptr))
						Globalinfo->bw_c_incoming_bytes += ret;
					else if (IsServer(clptr))
						Globalinfo->bw_s_incoming_bytes += ret;
				}
			}
			if (IsCClient(cptr))
			{
				IRCD_CLIENT *clptr = (IRCD_CLIENT *)cptr->conn_ptr;

				if ((IsClient(clptr) && (clptr->c_queue_in.queue_size > 8192)) ||
					(!IsServer(clptr) && (clptr->c_queue_in.queue_size > 32768*2)))
				{
					conn_del(meptr, cptr, "Excess Flood");
					continue;
				}
				if (!(clptr->c_flags &
						(CLIENT_FLAGS_DOINGAUTH|CLIENT_FLAGS_DOINGDNS)))
				{
					if (IsServer(clptr) &&
						CliSentEOB(clptr) && !CliGotEOB(clptr))

					{
						if ((Now - clptr->c_lastparse) > 4)
						{
							if (ircd_parse(clptr) < 0)
							{
								continue;
							}
							else
								clptr->c_lastparse = Now;
						}
					}
					else if (ircd_parse(clptr) < 0)
						continue;
					else
						clptr->c_lastparse = Now;
				}
			}
		}
		if (_pfdarray[i].revents & POLLOUT)
		{
			if (IsAuth(cptr))
			{
				IRCD_AUTH	*aptr = (IRCD_AUTH *)cptr->conn_ptr;
				char		buffer[64];
				int			len;
				int			sh = (int)vars_value_get_bynum(IRCD_VAR_SHOW_HEADERS);

				if (sh)
					send_to_client(aptr->a_clptr, "NOTICE * :*** Sending ident request...");
				sprintf(buffer, "%u , %u\r\n",
						(unsigned int)aptr->a_clptr->c_rem_port,
						(unsigned int)aptr->a_clptr->c_loc_port);
				len = strlen(buffer);
				if (write(cptr->fdlistinfo.fd, buffer, len) != len)
				{
					IRCD_CONN *cptrkeep = aptr->a_clptr->c_connptr;
					conn_del(meptr, cptr, "write error");
					cptr = cptrkeep; /* fall through */
					if (sh)
						send_to_client((IRCD_CLIENT *)cptr->conn_ptr, "NOTICE * :*** Ident request failed.");
					if (IsCClient(cptr))
					{
						IRCD_CLIENT	*clptr = (IRCD_CLIENT *)cptr->conn_ptr;


						if (!IsDoingDns(clptr))
						{
							if (ircd_parse(clptr) < 0)
							{
								continue;
							}
							else
								clptr->c_lastparse = Now;
						}
						continue;
					}
				}
				else
				{
					fdlist_del_event(cptr->fdlistinfo.fdlist,
						cptr->fdlistinfo.fdlist_pos,
						FDLIST_FLAGS_WRITE);
					continue;
				}
			}
			if (IsCClient(cptr))
			{
				IRCD_CLIENT	*clptr = (IRCD_CLIENT *)cptr->conn_ptr;

				if (clptr->c_flags & CLIENT_FLAGS_DOINGCONNECT)
				{
					clptr->c_flags &= ~CLIENT_FLAGS_DOINGCONNECT;
					if (!clptr->c_queue_out.queue_size) /* should be false */
						fdlist_del_event(cptr->fdlistinfo.fdlist,
							cptr->fdlistinfo.fdlist_pos,
							FDLIST_FLAGS_WRITE);
				}
				if ((clptr->c_maxsendq != -1) &&
					(clptr->c_queue_out.queue_size > clptr->c_maxsendq))
				{
					sprintf(exitbuf, "Max SendQ Exceeded: %i > %i",
							clptr->c_queue_out.queue_size, clptr->c_maxsendq);
					conn_del(meptr, cptr, exitbuf);
					continue;
				}
				if (client_queue_flush(clptr, exitbuf))
					conn_del(meptr, cptr, exitbuf);
			}
		}
	}
	return 0;
}

static void flood_check(void)
{
	IRCD_CLIENT	*fptr;
	IRCD_CLIENT	*nextf;

	for(;;)
	{
		Flooders_did_delete = 0;
		for(fptr=*Flooders;fptr;fptr=nextf)
		{
			nextf = fptr->c_f_next;
			if (Now > fptr->c_rate_time)
			{
				ircd_parse(fptr);
				if (Flooders_did_delete) /* dunno if "nextf" is OK now */
					break;
			}
		}
		if (!fptr)
			break;
	}
}

static void dns_check(void)
{
	IRCDRES_REQLIST	*rl;
	IRCDRES_REQLIST	*nextrl;
	int				ret;

	for(rl=Reqlist;rl;rl=nextrl)
	{
		nextrl = rl->rl_next;

		if (Now > rl->rl_firsttime + 15)
		{
			ircdres_request_del(rl->rl_req, 1); /*call callback w/ failure*/
			continue;
		}

		if (Now > rl->rl_lasttime + (rl->rl_req->req_sends * 2))
		{
			ret = ircdres_request_resend(rl->rl_req);
			if (ret < 0)
				continue;
			rl->rl_lasttime = Now;
		}
	}
}

static void auth_check(void)
{
	register IRCD_AUTH	*aptr;
	register IRCD_AUTH	*nextauth;
	int					sh = (int)vars_value_get_bynum(IRCD_VAR_SHOW_HEADERS);

	/* check Auths first. */
	for(;;)
	{
		Auths_did_delete = 0;
		for(aptr=Auths;aptr;aptr=nextauth)
		{
			nextauth = aptr->a_next;

			if (Now > aptr->a_firsttime + 10)
			{
				IRCD_CLIENT	*clptr = aptr->a_clptr;

				if (sh)
					send_to_client(clptr, "NOTICE * :*** Ident request failed: Timed out");
				auth_del(meptr, aptr, "Auth timeout.");
				if (!IsDoingDns(clptr))
				{
					if (ircd_parse(clptr) >= 0)
						clptr->c_lastparse = Now;
					if (Auths_did_delete)
						break;
				}
			}
		}
		if (!aptr)
			break;
	}
}

static void ping_check(void)
{
	register IRCD_CLIENT	*pptr;
	register IRCD_CLIENT	*nextlo;
	int						sh = (int)vars_value_get_bynum(IRCD_VAR_SHOW_HEADERS);

	for(pptr=Locals;pptr;pptr=nextlo)
	{
		nextlo = pptr->c_lo_next;

		if (IsDead(pptr))
		{
			client_del(meptr, pptr, "Dead client");
			continue;
		}

		if (IsUnknown(pptr) && (Now > pptr->c_firsttime + 30))
		{
			if (IsDoingConnect(pptr) ||
				(!IsDoingDns(pptr) && !IsDoingAuth(pptr) &&
						(Now > pptr->c_firsttime + 60)))
			{
				client_del(meptr, pptr, "Too much time as unknown");
				continue;
			}
			if (IsDoingDns(pptr))
			{
				int		type;

				type = pptr->c_res_req->rl_req->req_qtype;

				/*
				** can't rely on having this call the callback...
				** because the callback may remove "nextlo"
				*/
				ircdres_request_del(pptr->c_res_req->rl_req, 0);
				if (sh)
					send_to_client(pptr, "NOTICE * :*** %s DNS request failed: Timed out",
							type == T_PTR ? "Reverse" : "Forward");
				pptr->c_res_req->rl_req = NULL;
				del_reqlist_from_list(&Reqlist, pptr->c_res_req);
				pptr->c_flags &= ~CLIENT_FLAGS_DOINGDNS;
				if (!IsDoingAuth(pptr))
				{
					ircd_parse(pptr);
					/*
					** hate to start loop over, but I have to :(
					**  we can't guarantee that ->next is valid
					*/
					nextlo = Locals;
					continue;
				}
			}
		}

		if (IsDoingDns(pptr) || IsDoingAuth(pptr))
			continue;

		if (pptr->c_pingfreq && ((Now - pptr->c_lasttime) > pptr->c_pingfreq))
		{
			if (pptr->c_flags & CLIENT_FLAGS_PINGSENT)
			{
				if ((Now - pptr->c_lasttime) > (pptr->c_pingfreq*2))
				{
					char buffer[128];

					sprintf(buffer,
						"Ping timeout: no data for %i seconds",
						Now - pptr->c_lasttime);

					client_del(meptr, pptr, buffer);
					continue;
				}
			}
			else
			{
				send_to_client(pptr, "PING :%s", meptr->name);
				pptr->c_flags |= CLIENT_FLAGS_PINGSENT;
			}
		}
		/* fdlist check */
		if (IsClient(pptr))
		{
			int		fdlistnum = Now - pptr->c_idletime;
			FDLIST	*fdlist = Fdlist_main;
			int		mfd;
			int		ret;
			int		flags;
			int		nfdlists;
			int		ptime;

			if (!IsAnOper(pptr) && ((nfdlists = (int)vars_value_get_bynum(IRCD_VAR_PRIORITY_FDLISTS)) > 0))
			{
				ptime = (int)vars_value_get_bynum(IRCD_VAR_PRIORITY_IDLE);
				fdlistnum /= ptime;
				if (fdlistnum > 0)
				{
					fdlistnum--;
					if (fdlistnum >= nfdlists)
						fdlistnum = nfdlists - 1;
					fdlist = &Fdlist_clients[fdlistnum];
				}
			}
			if (pptr->c_connptr->fdlistinfo.fdlist != fdlist)
			{
				flags = fdlist_get_events(pptr->c_connptr->fdlistinfo.fdlist,
					pptr->c_connptr->fdlistinfo.fdlist_pos);
				assert(flags);
				ret = fdlist_del_fd(pptr->c_connptr->fdlistinfo.fdlist,
                        pptr->c_connptr->fdlistinfo.fdlist_pos, &mfd);
				if (mfd >= 0)
				{
					if (GET_FDS(mfd))
						GET_FDS(mfd)->fdlistinfo.fdlist_pos =
	                        pptr->c_connptr->fdlistinfo.fdlist_pos;
				}
				ret = ircd_open_fd(pptr->c_connptr->fdlistinfo.fd, flags, pptr->c_connptr, fdlist);
				if (ret < 0)
				{
					client_del(meptr, pptr, "Fdlist problem");
				}
			}
		}
	}
}

static void kline_check(void)
{
	register IRCD_CLIENT		*pptr;
	register IRCD_CLIENT		*nextlo;
	int				ret;
	char			exitbuf[512];
	CONFHOSTIPENTRY	*entryptr;
	char			cnamebuf[CLIENTNAMESIZE];

	for(pptr=Locals;pptr;pptr=nextlo)
	{
		nextlo = pptr->c_lo_next;

		if (!IsClient(pptr))
			continue;

		if (pptr->c_class->flags & CLASS_FLAGS_BYPASSDENY)
			continue;

		ret = access_match_ipaddr(&(_ircd_conf.deny),
				pptr->username, pptr->c_ipstring,
				pptr->c_rem_ipaddr, ACCESS_FLAGS_BESTMATCH,
				&entryptr);
		if (ret < 0)
			ret = access_match_hostname(&(_ircd_conf.deny),
				pptr->username, pptr->hostname, ACCESS_FLAGS_BESTMATCH,
				&entryptr);

		if (ret >= 0) /* banned */
		{
			send_to_opers("Kill line active for %s",
					ircd_clientname(pptr, 0, cnamebuf));
			send_prefix_to_client(meptr, pptr,
					IRCD_REPLY(ERR_YOUREBANNEDCREEP, pptr->name),
					entryptr->data ? entryptr->data : "No reason specified");
			sprintf(exitbuf, "Banned: %s",
				entryptr->data ? entryptr->data : "No reason specified");
			client_del(meptr, pptr, exitbuf);
			continue;
		}
	}
}

static void rejecthold_check(void)
{
	IRCD_REJECTHOLD		*rptr;
	IRCD_REJECTHOLD		*nextrptr;
	int					num = (int)vars_value_get_bynum(IRCD_VAR_REJECT_HOLD);

	for(rptr=Rejectholds;rptr;rptr=rptr=nextrptr)
	{
		nextrptr = rptr->rh_next;
		if ((num <= 0) || ((rptr->rh_firsttime + num) <= Now))
		{
			rejecthold_del(meptr, rptr, "bye now");
		}
	}
}

static int autoconn_check(void)
{
	SERVLINK		*s;
	SERVLINK		*s2;
	IRCD_CLIENT		*sptr;
	IRCD_CLASSENTRY	*classptr;
	int				ret;
	int				nextcheck = 300;
	int				diff;

	for(s=_ircd_conf.servers;s;s=s->next)
	{
		if (!(s->flags & SERVLINK_FLAGS_AUTOCONN))
			continue;
		if (Now < s->tryconnecttime + 60)
		{
			continue;
		}
		sptr = hash_find_client_incl_hm(s->servername);
		if (!sptr)
		{
			/*
			** ok, this one doesn't exist...let's see if another one in
			**	the same class does exist...
			*/
			if (s->classptr)
				classptr = (IRCD_CLASSENTRY *)s->classptr;
			else
			{
				ret = class_find(&(_ircd_conf.classes), s->classname, &classptr);

				if (ret)
					continue;
				s->classptr = (void *)classptr;
			}
			for(sptr=classptr->clients;sptr;sptr=sptr->c_cl_next)
				if (IsServer(sptr))
					break;
			if (sptr)		/* yup!, don't autoconn this one */
				continue;

			if ((diff = classptr->lastconnect + classptr->connfreq) <= Now)
			{
				int ret2;

				for(s2=_ircd_conf.servers;s2;s2=s2->next)
				{
					if (!s2->classptr)
					{
						IRCD_CLASSENTRY *classptr2;

						ret = class_find(&(_ircd_conf.classes),
										s2->classname, &classptr2);
						if (ret)
							continue;
						s2->classptr = (void *)classptr2;
					}
					if (s2 == s)
						continue;
					if ((s2->classptr == s->classptr) &&
						(Now < s2->tryconnecttime + 60))
							break;
				}
				if (s2)
					continue; /* already trying to connect to another */
				ret2 = _do_connect(NULL, s, -1);

				classptr->lastconnect = Now;
				if (classptr->connfreq < nextcheck)
					nextcheck = classptr->connfreq;

				if (!ret2)
				{
					servlink_rotate(s);
					break;
				}
			}
			else if (diff - Now < nextcheck)
				nextcheck = diff - Now;
		}
		else
		{
			if (s->classptr)
				classptr = (IRCD_CLASSENTRY *)s->classptr;
			else
			{
				ret = class_find(&(_ircd_conf.classes), s->classname, &classptr);

				if (ret)
					continue;
				s->classptr = (void *)classptr;
			}
			if (classptr->connfreq < nextcheck)
				nextcheck = classptr->connfreq;
		}
	}
	return nextcheck < 10 ? 10 : nextcheck;
}

static int _ircdso_firsttime(char *conf_file)
{
	int				ret;
	register int	i;
	int				opt;
	int				optlen;

	Ircd_zero_fd = open("/dev/zero", O_RDWR);
	if (Ircd_zero_fd < 0)
	{
		fprintf(stderr, "Couldn't open /dev/zero\n");
		exit(-1);
	}

	if ((i = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr, "couldn't socket(): %i\n", -errno);
		exit(-1);
	}

	optlen = sizeof(opt);
	if (getsockopt(i, SOL_SOCKET, SO_RCVBUF, (char *)&opt, &optlen) < 0)
	{
		fprintf(stderr, "couldn't getsockopt(): %i\n", -errno);
		exit(-1);
	}

	while(setsockopt(i, SOL_SOCKET, SO_RCVBUF, (char *)&opt, sizeof(opt)) >= 0)
	{
		opt += 1024;
		if (opt > (64*1024))
			break;
	}

	optlen = sizeof(opt);
	if (getsockopt(i, SOL_SOCKET, SO_RCVBUF, (char *)&opt, &optlen) < 0)
	{
		fprintf(stderr, "couldn't getsockopt(): %i\n", -errno);
		exit(-1);
	}

	Recvbuffersize = opt;

	optlen = sizeof(opt);
	if (getsockopt(i, SOL_SOCKET, SO_SNDBUF, (char *)&opt, &optlen) < 0)
	{
		fprintf(stderr, "couldn't getsockopt(): %i\n", -errno);
		exit(-1);
	}

	while(setsockopt(i, SOL_SOCKET, SO_SNDBUF, (char *)&opt, sizeof(opt)) >= 0)
	{
		opt += 1024;
		if (opt > (64*1024))
			break;
	}

	optlen = sizeof(opt);
	if (getsockopt(i, SOL_SOCKET, SO_SNDBUF, (char *)&opt, &optlen) < 0)
	{
		fprintf(stderr, "couldn't getsockopt(): %i\n", -errno);
		exit(-1);
	}

	Sendbuffersize = opt;
	close(i);

	Conffile = MyStrdup(conf_file);

	meptr = (IRCD_CLIENT *)MyMalloc(sizeof(IRCD_CLIENT));
	Fds_size = 1024;
	Fds = (IRCD_CONN **)MyMalloc(sizeof(IRCD_CONN *) * Fds_size);
	Sentalong_size = 1024;
	Sentalong = (int *)MyMalloc(sizeof(int) * Sentalong_size);
	Sentalong_marker = 0;
	Sentalong_end = Sentalong + Sentalong_size;
	Globalinfo = (GLOBALSTATS *)MyMalloc(sizeof(GLOBALSTATS));
	Globalinfo->m_maxconntime = Globalinfo->g_maxclientstime = 
			Globalinfo->g_maxserverstime = Globalinfo->maxchannelstime =
			Globalinfo->m_maxclientstime = Globalinfo->m_maxserverstime =
			Globalinfo->m_totclientstime = (int)time(NULL);
	Globalinfo->g_servers = 1;

	Fdlist_num = 100;
	Fdlist_main = (FDLIST *)MyMalloc(sizeof(FDLIST));
	Fdlist_clients = (FDLIST *)MyMalloc(sizeof(FDLIST) * Fdlist_num);
	Fdlist_lasttime = (int *)MyMalloc(sizeof(int) * Fdlist_num);

	Flooders = (IRCD_CLIENT **)MyMalloc(sizeof(IRCD_CLIENT *));
	Clients = (IRCD_CLIENT **)MyMalloc(sizeof(IRCD_CLIENT *));
	Opers = (IRCD_CLIENT **)MyMalloc(sizeof(IRCD_CLIENT *));
	Servers = (IRCD_CLIENT **)MyMalloc(sizeof(IRCD_CLIENT *));

	Ircd_resconn = (IRCD_CONN *)MyMalloc(sizeof(IRCD_CONN));
	Ircd_resconn->conn_type = IRCD_CONN_TYPE_RES;

	ret = ircdres_init((IRCDRES **)(&(Ircd_resconn->conn_ptr)));
	if (ret < 0)
	{
		fprintf(stderr, "Couldn't do ircdres_init: %i\n", ret);
		exit(-1);
	}

#if 0
	ircd_pagesize = ircd_getpagesize();
#else
	ircd_pagesize = 2*1024;
#endif

	vars_init();
	numeric_init();
	ircdconf_init(&_ircd_conf);
	fdlist_init(Fdlist_main);

	for(i=0;i<Fdlist_num;i++)
	{
		fdlist_init(&Fdlist_clients[i]);
		Fdlist_lasttime[i] = 0;
	}

	iphash_init();

	if (ircd_open_fd(((IRCDRES *)Ircd_resconn->conn_ptr)->res_fd,FDLIST_FLAGS_READ, Ircd_resconn, NULL)<0)
	{
		fprintf(stderr, "Error putting resfd into fdlist\n");
		exit(-1);
	}

	Now = (int)time(NULL);
	ret = ircdconf_read(Conffile, &_ircd_conf);
	if (ret < 0)
	{
		fprintf(stderr, "Couldn't read config file \"%s\": %i\n",
					Conffile, ret);
		exit(ret);
	}
	if (!_ircd_conf.name)
	{
		fprintf(stderr, "No server name given in ircd.conf\n");
		exit(-1);
	}

	{
		char    *motd_file = vars_value_get_bynum(IRCD_VAR_MOTD_FILE);

		if (motd_file)
			motd_load(motd_file, &Motd);
	}

	whowas_init();
	support_strncpy(meptr->name, _ircd_conf.name, sizeof(meptr->name));
	support_strncpy(meptr->ircname, _ircd_conf.info ? _ircd_conf.info : "Unknown",
			sizeof(meptr->ircname));
	meptr->c_type = CLIENT_TYPE_SERVER;
	meptr->c_from = meptr;
	meptr->c_servptr = meptr;
	meptr->c_firsttime = meptr->c_lasttime = Now;
	hash_add_client(meptr);
	add_client_to_oslist(Servers, meptr);

	if (!_ircd_conf.listeners)
	{
		fprintf(stderr, "No ports on which to listen\n");
		exit(-1);
	}

	fclose(stdin);
	fclose(stdout);

#ifdef TIOCNOTTY
	{
		int fd = open("/dev/tty", O_RDWR);

		if (fd >= 0)
		{
			ioctl(fd, TIOCNOTTY, NULL);
			close(fd);
		}
	}
#endif

	logs_init();
	syslog(LOG_NOTICE, "ircd has started.");

	{
		int fd = open("csircd.pid", O_CREAT|O_TRUNC|O_WRONLY, 0660);

		if (fd >= 0)
		{
			char buf[10];

			sprintf(buf, "%i\n", (int)getpid());
			write(fd, buf, strlen(buf));
			close(fd);
		}
	}
	return 0;
}

static int _ircdso_init(int firsttime)
{
	int			needconversion;
	IRCD_DATA	*Coreptr = *((IRCD_DATA **)ircdcore_data);

	needconversion = 0;

	signal(SIGHUP, _ircd_sighup_handler);
	signal(SIGUSR1, _ircd_sigusr1_handler);

	command_tree_init();
	hash_init();

	if (firsttime)
	{
		return _ircdso_firsttime(Coreptr->Conffile);
	}
	else if (needconversion)
	{
		;
	}
	else
	{
		if (!Coreptr || (*ircdcore_data_size != sizeof(IRCD_DATA)))
			return -EFAULT;
		Fdlist_num = Coreptr->Fdlist_num;
		Conffile = Coreptr->Conffile;
		ircd_pagesize = Coreptr->ircd_pagesize;
		Fdlist_main = Coreptr->Fdlist_main;
		Fdlist_clients = Coreptr->Fdlist_clients;
		Fdlist_lasttime = Coreptr->Fdlist_lasttime;
		Ircd_resconn = Coreptr->Ircd_resconn;
		meptr = Coreptr->meptr;
		Fds = Coreptr->Fds;
		Fds_size = Coreptr->Fds_size;
		Reqlist = Coreptr->ResReqlist;
		Locals = Coreptr->Locals;
		Auths = Coreptr->Auths;
		Clients = Coreptr->Clients;
		Opers = Coreptr->Opers;
		Servers = Coreptr->Servers;
		Flooders = Coreptr->Flooders;
		Channels = Coreptr->Channels;
		Globalinfo = Coreptr->Globalinfo;
		Rejectholds = Coreptr->Rejectholds;
		Motd = Coreptr->Motd;
		Sendbuffersize = Coreptr->Sendbuffersize;
		Recvbuffersize = Coreptr->Recvbuffersize;
		Sentalong = Coreptr->Sentalong;
		Sentalong_marker = Coreptr->Sentalong_marker;
		Sentalong_end = Coreptr->Sentalong_end;
		Sentalong_size = Coreptr->Sentalong_size;
		Ircd_zero_fd = Coreptr->Ircd_zero_fd;
		BCOPY(&(Coreptr->Ircd_conf), &_ircd_conf, sizeof(_ircd_conf));
		queue_set_stats(Coreptr->queue_size, Coreptr->queue_malloced_size);
		ircdres_init_reload(Coreptr->ResRequests);
		numeric_init_reload(Coreptr->NumericArr, Coreptr->NumericArrSize);
		whowas_init_reload(Coreptr->whowasarr, Coreptr->whowas_size,
			Coreptr->whowashash, Coreptr->whowas_hash_size,
			Coreptr->whowas_next);
		vars_init_reload(Coreptr->Vars);
		hash_init_reload(Coreptr->client_hash, Coreptr->client_hash_size,
			Coreptr->channel_hash, Coreptr->channel_hash_size);
		iphash_init_reload(Coreptr->iphash, Coreptr->iphash_size,
				Coreptr->clones, Coreptr->throttles);
	}

	DoRehash = 0;
	RehashWho = NULL;
	Rehashed = 0;
	Autoconncheck = 0;

	return 0;
}

static int _ircdso_deinit()
{
	IRCD_DATA	*Coreptr = *((IRCD_DATA **)ircdcore_data);


	if (*ircdcore_data)
		MyFree(*ircdcore_data);
	*ircdcore_data = (char *)MyMalloc(sizeof(IRCD_DATA));
	Coreptr = *((IRCD_DATA **)ircdcore_data);

	*ircdcore_data_size = sizeof(IRCD_DATA);
	*ircdcore_data_format_version = 0;

	if (_pfdarray)
		MyFree(_pfdarray);

	ircdres_deinit_reload(&Coreptr->ResRequests);
	Coreptr->Conffile = Conffile;
	Coreptr->ircd_pagesize = ircd_pagesize;
	Coreptr->Fdlist_num = Fdlist_num;
	Coreptr->Fdlist_main = Fdlist_main;
	Coreptr->Fdlist_clients = Fdlist_clients;
	Coreptr->Fdlist_lasttime = Fdlist_lasttime;
	Coreptr->Ircd_resconn = Ircd_resconn;
	Coreptr->meptr = meptr;
	Coreptr->Fds = Fds;
	Coreptr->Fds_size = Fds_size;
	Coreptr->ResReqlist = Reqlist;
	Coreptr->Locals = Locals;
	Coreptr->Auths = Auths;
	Coreptr->Clients = Clients;
	Coreptr->Opers = Opers;
	Coreptr->Servers = Servers;
	Coreptr->Flooders = Flooders;
	Coreptr->Channels = Channels;
	Coreptr->Globalinfo = Globalinfo;
	Coreptr->Rejectholds = Rejectholds;
	Coreptr->Motd = Motd;
	Coreptr->Sendbuffersize = Sendbuffersize;
	Coreptr->Recvbuffersize = Recvbuffersize;
	Coreptr->Sentalong = Sentalong;
	Coreptr->Sentalong_size = Sentalong_size;
	Coreptr->Sentalong_marker = Sentalong_marker;
	Coreptr->Sentalong_end = Sentalong_end;
	Coreptr->Ircd_zero_fd = Ircd_zero_fd;
	queue_get_stats(&Coreptr->queue_size, NULL, &Coreptr->queue_malloced_size);
	BCOPY(&_ircd_conf, &(Coreptr->Ircd_conf), sizeof(_ircd_conf));
	numeric_deinit_reload(&Coreptr->NumericArr, &Coreptr->NumericArrSize);
	whowas_deinit_reload(&Coreptr->whowasarr, &Coreptr->whowas_size,
		&Coreptr->whowashash, &Coreptr->whowas_hash_size,
		&Coreptr->whowas_next);
	vars_deinit_reload(&Coreptr->Vars);
	hash_deinit_reload(&Coreptr->client_hash, &Coreptr->client_hash_size,
		&Coreptr->channel_hash, &Coreptr->channel_hash_size);
	iphash_deinit_reload(&Coreptr->iphash, &Coreptr->iphash_size,
			&Coreptr->clones, &Coreptr->throttles);

	if (RehashWho)
		MyFree(RehashWho);

	command_tree_free();

	signal(SIGHUP, SIG_IGN);
	signal(SIGUSR1, SIG_IGN);

	closelog();

	return 0;
}


int _ircd_main_loop(int firsttime,
					char *Ircdcore_versionstring,
					int	*Data_format_version,
					char **Data,
					int *Data_size)
{
	int			ret;
	FDLIST		*curfdlist;
	int			i;

	if (!Data_format_version || !Data || !Data_size ||
			!Ircdcore_versionstring)
		return -EINVAL;


	ircdcore_versionstring = Ircdcore_versionstring;
	ircdcore_data_format_version = Data_format_version;
	ircdcore_data = Data;
	ircdcore_data_size = Data_size;

	openlog("csircd", LOG_PID|LOG_NDELAY, LOG_DAEMON);

	if (*ircdcore_data_format_version > 0)
		return -EINVAL;

	ret = _ircdso_init(firsttime);
	if (ret < 0)
		return ret;

	if (!firsttime)
		send_to_opers("the csircd library has been reloaded");

	curfdlist = Fdlist_main;
	for(;;)
	{
		if (DoRehash)
		{
			ircdconf_rehash();
			DoRehash = 0;
			Autoconncheck = 0;
		}

		if (DoReload)
		{
			ret = _ircdso_deinit();

			if (ret < 0)
			{
				send_to_opers("Couldn't prepare for csircd library unload: %i", ret);
			}
			else
			{
				return 0;
			}
		}

		ret = _ircd_pollfds(curfdlist, 2);

		if (Now >= (Fdlist_lasttime[0] + (int)vars_value_get_bynum(IRCD_VAR_PRIORITY_LAG)))
		{
			_ircd_pollfds(&Fdlist_clients[0], 0);
			Fdlist_lasttime[0] = Now;
			for(i=1;i<(int)vars_value_get_bynum(IRCD_VAR_PRIORITY_FDLISTS);i++)
			{
				if (Now >= (Fdlist_lasttime[i] + ((i+1)*(int)vars_value_get_bynum(IRCD_VAR_PRIORITY_LAG))))
				{
					_ircd_pollfds(&Fdlist_clients[i], 0);
					Fdlist_lasttime[i] = Now;
				}
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 2))
			{
				flood_check();
				lastcheck = Now;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 2))
			{
				dns_check();
				lastcheck = Now;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 2))
			{
				auth_check();
				lastcheck = Now;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 15))
			{
				ping_check();
				lastcheck = Now;
			}
		}

		if (Now >= (Globalinfo->bw_last_check + 5))
		{
			if (Globalinfo->bw_last_check)
			{
				Globalinfo->bw_c_incoming_rate = Globalinfo->bw_c_incoming_bytes/(Now - Globalinfo->bw_last_check);
				Globalinfo->bw_c_outgoing_rate = Globalinfo->bw_c_outgoing_bytes/(Now - Globalinfo->bw_last_check);
				Globalinfo->bw_s_incoming_rate = Globalinfo->bw_s_incoming_bytes/(Now - Globalinfo->bw_last_check);
				Globalinfo->bw_s_outgoing_rate = Globalinfo->bw_s_outgoing_bytes/(Now - Globalinfo->bw_last_check);
			}
			Globalinfo->bw_c_incoming_bytes = 0;
			Globalinfo->bw_c_outgoing_bytes = 0;
			Globalinfo->bw_s_incoming_bytes = 0;
			Globalinfo->bw_s_outgoing_bytes = 0;
			Globalinfo->bw_last_check = Now;
		}

		{
			static int	lastcheck = 0;

			if (Rehashed || (Now >= (lastcheck + 900)))
			{
				kline_check();
				lastcheck = Now;
				Rehashed = 0;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 60))
			{
				access_flush(&_ircd_conf.deny, 0);
				lastcheck = Now;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 10))
			{
				iphash_cleanup();
				lastcheck = Now;
			}
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 900))
			{
				queue_cleanup();
				lastcheck = Now;
			}	
		}

		{
			static int	lastcheck = 0;

			if (Now >= (lastcheck + 10))
			{
				rejecthold_check();
				lastcheck = Now;
			}
		}

		if (Now >= Autoconncheck)
		{
			Autoconncheck = Now + autoconn_check();
		}
	}

	return 0;
}
