/*
**   IRC - Internet Relay Chat, source/log.c
**   Copyright (C) 1999 Chris Behrens
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ircd_includes.h"
#include "log.h"

RCSID("$Id: log.c,v 1.19 2001/09/30 11:02:14 cbehrens Exp $");

static void _log_close(aLog *log);

static void _log_close(aLog *log)
{
	if (log && log->fb)
	{
		fbclose(log->fb);
		log->fb = NULL;
	}
}

int log_samename(aLog *log, char *filename)
{
	return log && log->filename ? !strcmp(log->filename, filename) : 0;
}

aLog *log_open(char *filename)
{
	aLog	*log_new;

	log_new = (aLog *)MyMalloc(sizeof(aLog));
	log_new->filename = MyStrdup(filename);
	log_new->last_attempt = 0;
	return log_new;
}

void log_close(aLog *log)
{
	if (log)
	{
		if (log->fb)
			fbclose(log->fb);
		if (log->filename)
			MyFree(log->filename);
		MyFree(log);
	}
}

void log_flush(aLog *log)
{
	if (log && log->fb)
		fbflush(log->fb);
}

int log_printf(aLog *log, char *format, ...)
{
	va_list	ap;
	char	buffer[2048];
	int		ret;

	if (!log)
		return -1;
	if (log->fb == NULL)
	{
		if (log->last_attempt + 60 <= Now)
		{
			log->fb = fbopen(log->filename, "a");
			log->last_attempt = Now;
		}
		if (log->fb == NULL)
			return 0;
	}
	va_start(ap, format);
#ifdef HAVE_VSNPRINTF
	ret = vsnprintf(buffer, sizeof(buffer)-1, format, ap);
#else
	ret = vsprintf(buffer, format, ap);
#endif
	va_end(ap);
	if (ret >= sizeof(buffer))
		ret = sizeof(buffer)-1;
	buffer[ret] = '\0';
	ret = fbwrite(log->fb, buffer, ret);
	if (ret <= 0)
		_log_close(log);
	return 0;
}

int log_write(aLog *log, char *buffer, int size)
{
	int	ret;

	if (!log)
		return -1;
	if (log->fb == NULL)
	{
		if (log->last_attempt + 60 <= Now)
		{
			log->fb = fbopen(log->filename, "a");
			log->last_attempt = Now;
		}
		if (log->fb == NULL)
			return 0;
	}
	ret = fbwrite(log->fb, buffer, size);
	if (ret <= 0)
		_log_close(log);
	return ret;
}

