/*
**   IRC - Internet Relay Chat, source/numeric.c
**   Copyright (C) 1999 Chris Behrens
**
**   Portions Copyright (C) 1990 Jarkko Oikarinen and
**                      University of Oulu, Computing Center
**        and Copyright (C) 1992 Darren Reed
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ircd_includes.h"
#include "numeric.h"

RCSID("$Id: numeric.c,v 1.19 2001/09/30 11:02:15 cbehrens Exp $");


typedef struct _ircd_reply	IRCD_REPLY;

struct _ircd_reply
{
    int     num_val;
    char    *num_form;
};

static IRCD_REPLY _ircd_lreplies[] =
{
/* 000 */	{	0,						(char *)NULL },
/* 001 */	{	RPL_WELCOME,
				":Welcome to the Internet Relay Network %s" },
/* 002 */	{	RPL_YOURHOST,
				":Your host is %s, running version %s" },
/* 003 */	{	RPL_CREATED,			":This server was created %s" },
/* 004 */	{	RPL_MYINFO,				"%s %s oOi biklmnopstv" },
			{	0,						(char *)NULL }
};

static IRCD_REPLY _ircd_replies[] = 
{
/* 200 */	{	RPL_TRACELINK,			"Link %s%s %s %s" },
/* 201 */	{	RPL_TRACECONNECTING,	"Try. %s %s" },
/* 202 */	{	RPL_TRACEHANDSHAKE,		"H.S. %s %s" },
/* 203 */	{	RPL_TRACEUNKNOWN,		"???? %s %s" },
/* 204 */	{	RPL_TRACEOPERATOR,		"Oper %s %s" },
/* 205 */	{	RPL_TRACEUSER,			"User %s %s" },
/* 206 */	{	RPL_TRACESERVER,		"Serv %s %dS %dC %s %s" },
/* 207 */	{	RPL_TRACESERVICE,		"Service %s %s" },
/* 208 */	{	RPL_TRACENEWTYPE,		"<newtype> 0 %s" },
/* 209 */	{	RPL_TRACECLASS,			"Class %s %d" },
/* 210 */	{	0,						(char *)NULL },
/* 211 */	{	RPL_STATSLINKINFO,		(char *)NULL },
/* 212 */	{	RPL_STATSCOMMANDS,		"%s %u %u" },
/* 213 */	{	RPL_STATSCLINE,			"%c %s@%s * %s %d %s %s" },
/* 214 */	{	RPL_STATSNLINE,			"%c %s@%s * %s %d %s %s" },
/* 215 */	{	RPL_STATSILINE,			"%c %s * %s %d %s" },
/* 216 */	{	RPL_STATSKLINE,			"%c %s * %s %d %d :%s" },
/* 217 */	{	RPL_STATSQLINE,			"%s %s@%s %d %d :%s" },
/* 218 */	{	RPL_STATSYLINE,			"%c %s %d %d %d %ld %d %d %d %d %d %d %d" },
/* 219 */	{	RPL_ENDOFSTATS,			"%c :End of /STATS report" },
/* 220 */	{	0,						(char *)NULL },
/* 221 */	{	RPL_UMODEIS,			"%s" },
/* 222 */	{	RPL_STATSBLINE,			"%c %s * %s %d %d" },
/* 223 */	{	RPL_STATSELINE,			"%c %s * %s %d %d" },
/* 224 */	{	RPL_STATSDLINE,			"%c %s * %s %d %d :%s" },
/* 225 */	{	RPL_STATSGLINE,			"%c %s %s %s %d %d" },
/* 226 */	{	0,						(char *)NULL },
/* 227 */	{	0,						(char *)NULL },
/* 228 */	{	0,						(char *)NULL },
/* 229 */	{	0,						(char *)NULL },
/* 230 */	{	0,						(char *)NULL },
/* 231 */	{	RPL_SERVICEINFO,		(char *)NULL },
/* 232 */	{	RPL_ENDOFSERVICES,		(char *)NULL },
/* 233 */	{	RPL_SERVICE,			(char *)NULL },
/* 234 */	{	RPL_SERVLIST,			(char *)NULL },
/* 235 */	{	RPL_SERVLISTEND,		(char *)NULL },
/* 236 */	{	0,						(char *)NULL },
/* 237 */	{	0,						(char *)NULL },
/* 238 */	{	0,						(char *)NULL },
/* 239 */	{	0,						(char *)NULL },
/* 240 */	{	0,						(char *)NULL },
/* 241 */	{	RPL_STATSLLINE,			"%c %s * %s %d %d" },
/* 242 */	{	RPL_STATSUPTIME,		":Server Up %d days, %d:%02d:%02d" },
/* 243 */	{	RPL_STATSOLINE,			"%c %s@%s * %s %d %s" },
/* 244 */	{	RPL_STATSHLINE,			"%c %s * %s %d %d" }, 
/* 245 */	{	RPL_STATSSLINE,			"%c %s * %s %d %d" }, 
/* 246 */	{	0,						(char *)NULL },
/* 247 */	{	RPL_STATSXLINE,			"%s %d %d :%s" },
/* 248 */	{	0,						(char *)NULL },
/* 249 */	{	0,						(char *)NULL },
/* 250 */	{	RPL_STATSCONN,
				":Highest connection count: %d (%d clients)" },
/* 251 */	{	RPL_LUSERCLIENT,
		":There are %d users and %d invisible on %d servers" },
/* 252 */	{	RPL_LUSEROP,			"%d :operator(s) online" },
/* 253 */	{	RPL_LUSERUNKNOWN,		"%d :unknown connection(s)" },
/* 254 */	{	RPL_LUSERCHANNELS,		"%d :channels formed" },
/* 255 */	{	RPL_LUSERME,			":I have %d clients and %d servers" },
/* 256 */	{	RPL_ADMINME,			":Administrative info about %s" },
/* 257 */	{	RPL_ADMINLOC1,			":%s" },
/* 258 */	{	RPL_ADMINLOC2,			":%s" },
/* 259 */	{	RPL_ADMINEMAIL,			":%s" },
/* 260 */	{	0,						(char *)NULL },
/* 261 */	{	RPL_TRACELOG,			"File %s %d" },
/* 262 */	{	RPL_ENDOFTRACE,			"%s :End of /TRACE list." },
/* 263 */	{	RPL_LOAD2HI,
				"%s :Server load too high, try again in a while" },
/* 264 */	{	0,						(char *)NULL },
/* 265 */	{	0,						(char *)NULL },
/* 266 */	{	0,						(char *)NULL },
/* 267 */	{	0,						(char *)NULL },
/* 268 */	{	0,						(char *)NULL },
/* 269 */	{	0,						(char *)NULL },
/* 270 */	{	0,						(char *)NULL },
/* 271 */	{	0,						(char *)NULL },
/* 272 */	{	0,						(char *)NULL },
/* 273 */	{	0,						(char *)NULL },
/* 274 */	{	0,						(char *)NULL },
/* 275 */	{	0,						(char *)NULL },
/* 276 */	{	0,						(char *)NULL },
/* 277 */	{	0,						(char *)NULL },
/* 278 */	{	0,						(char *)NULL },
/* 279 */	{	0,						(char *)NULL },
/* 280 */	{	0,						(char *)NULL },
/* 281 */	{	0,						(char *)NULL },
/* 282 */	{	0,						(char *)NULL },
/* 283 */	{	0,						(char *)NULL },
/* 284 */	{	0,						(char *)NULL },
/* 285 */	{	0,						(char *)NULL },
/* 286 */	{	0,						(char *)NULL },
/* 287 */	{	0,						(char *)NULL },
/* 288 */	{	0,						(char *)NULL },
/* 289 */	{	0,						(char *)NULL },
/* 290 */	{	0,						(char *)NULL },
/* 291 */	{	0,						(char *)NULL },
/* 292 */	{	0,						(char *)NULL },
/* 293 */	{	0,						(char *)NULL },
/* 294 */	{	0,						(char *)NULL },
/* 295 */	{	0,						(char *)NULL },
/* 296 */	{	0,						(char *)NULL },
/* 297 */	{	0,						(char *)NULL },
/* 298 */	{	0,						(char *)NULL },
/* 299 */	{	0,						(char *)NULL },
/* 300 */	{	RPL_NONE,				(char *)NULL },
/* 301 */	{	RPL_AWAY,				"%s :%s" },
/* 302 */	{	RPL_USERHOST,			":%s" },
/* 303 */	{	RPL_ISON,				":%s" },
/* 304 */	{	RPL_TEXT,				(char *)NULL },
/* 305 */	{	RPL_UNAWAY,
				":You are no longer marked as being away" },
/* 306 */	{	RPL_NOWAWAY,			":You have been marked as being away" },
/* 307 */	{	0,						(char *)NULL },
/* 308 */	{	0,						(char *)NULL },
/* 309 */	{	0,						(char *)NULL },
/* 310 */	{	0,						(char *)NULL },
/* 311 */	{	RPL_WHOISUSER,			"%s %s %s * :%s" },
/* 312 */	{	RPL_WHOISSERVER,		"%s %s :%s" },
/* 313 */	{	RPL_WHOISOPERATOR,		"%s :is an IRC Operator" },
/* 314 */	{	RPL_WHOWASUSER,			"%s %s %s * :%s" },
/* 315 */	{	RPL_ENDOFWHO,			"%s :End of /WHO list." },
/* 316 */	{	RPL_WHOISCHANOP,		(char *)NULL },
/* 317 */	{	RPL_WHOISIDLE,
				"%s %ld %ld :seconds idle, signon time" },
/* 318 */	{	RPL_ENDOFWHOIS,			"%s :End of /WHOIS list." },
/* 319 */	{	RPL_WHOISCHANNELS,		"%s :%s" },
/* 320 */	{	0,						(char *)NULL },
/* 321 */	{	RPL_LISTSTART,			"Channel :Users  Name" },
/* 322 */	{	RPL_LIST,				"%s %d :%s" },
/* 323 */	{	RPL_LISTEND,			":End of /LIST" },
/* 324 */	{	RPL_CHANNELMODEIS,		"%s %s %s" },
/* 325 */	{	0,						(char *)NULL },
/* 326 */	{	0,						(char *)NULL },
/* 327 */	{	0,						(char *)NULL },
/* 328 */	{	0,						(char *)NULL },
/* 329 */	{	RPL_CREATIONTIME,		"%s %u" },
/* 330 */	{	0,						(char *)NULL },
/* 331 */	{	RPL_NOTOPIC,			"%s :No topic is set." },
/* 332 */	{	RPL_TOPIC,				"%s :%s" },
/* 333 */	{	RPL_TOPICWHOTIME,		"%s %s %u" },
/* 334 */	{	0,						(char *)NULL },
/* 335 */	{	0,						(char *)NULL },
/* 336 */	{	0,						(char *)NULL },
/* 337 */	{	0,						(char *)NULL },
/* 338 */	{	0,						(char *)NULL },
/* 339 */	{	0,						(char *)NULL },
/* 340 */	{	0,						(char *)NULL },
/* 341 */	{	RPL_INVITING,			"%s %s" },
/* 342 */	{	RPL_SUMMONING,			"%s :User summoned to irc" },
/* 343 */	{	0,						(char *)NULL },
/* 344 */	{	0,						(char *)NULL },
/* 345 */	{	0,						(char *)NULL },
/* 346 */	{	0,						(char *)NULL },
/* 347 */	{	0,						(char *)NULL },
/* 348 */	{	0,						(char *)NULL },
/* 349 */	{	0,						(char *)NULL },
/* 350 */	{	0,						(char *)NULL },
/* 351 */	{	RPL_VERSION,			"%s.%s %s :%s" },
/* 352 */	{	RPL_WHOREPLY,			"%s %s %s %s %s %s :%d %s" },
/* 353 */	{	RPL_NAMREPLY,			"%s" },
/* 354 */	{	0,						(char *)NULL },
/* 355 */	{	0,						(char *)NULL },
/* 356 */	{	0,						(char *)NULL },
/* 357 */	{	0,						(char *)NULL },
/* 358 */	{	0,						(char *)NULL },
/* 359 */	{	0,						(char *)NULL },
/* 360 */	{	0,						(char *)NULL },
/* 361 */	{	RPL_KILLDONE,			(char *)NULL },
/* 362 */	{	RPL_CLOSING,			"%s :Closed. Status = %d" },
/* 363 */	{	RPL_CLOSEEND,			"%d: Connections Closed" },
/* 364 */	{	RPL_LINKS,				"%s %s :%d %s" },
/* 365 */	{	RPL_ENDOFLINKS,			"%s :End of /LINKS list." },
/* 366 */	{	RPL_ENDOFNAMES,			"%s :End of /NAMES list." },
/* 367 */	{	RPL_BANLIST,			"%s %s %s %u" },
/* 368 */	{	RPL_ENDOFBANLIST,		"%s :End of Channel Ban List" },
/* 369 */	{	RPL_ENDOFWHOWAS,		"%s :End of WHOWAS" },
/* 370 */	{	0,						(char *)NULL },
/* 371 */	{	RPL_INFO,				":%s" },
/* 372 */	{	RPL_MOTD,				":- %s" },
/* 373 */	{	RPL_INFOSTART,			":Server INFO" },
/* 374 */	{	RPL_ENDOFINFO,			":End of /INFO list." },
/* 375 */	{	RPL_MOTDSTART,			":- %s Message of the Day - " },
/* 376 */	{	RPL_ENDOFMOTD,			":End of /MOTD command." },
/* 377 */	{	0,						(char *)NULL },
/* 378 */	{	0,						(char *)NULL },
/* 379 */	{	0,						(char *)NULL },
/* 380 */	{	0,						(char *)NULL },
/* 381 */	{	RPL_YOUREOPER,			":You are now an IRC Operator" },
/* 382 */	{	RPL_REHASHING,			"%s :Rehashing" },
/* 383 */	{	RPL_YOURESERVICE,		(char *)NULL },
/* 384 */	{	RPL_MYPORTIS,			"%d :Port to local server is\r\n" },
/* 385 */	{	RPL_NOTOPERANYMORE,		(char *)NULL },
/* 386 */	{	0,						(char *)NULL },
/* 387 */	{	0,						(char *)NULL },
/* 388 */	{	0,						(char *)NULL },
/* 389 */	{	0,						(char *)NULL },
/* 390 */	{	0,						(char *)NULL },
/* 391 */	{	RPL_TIME,				"%s :%s" },
#ifdef	ENABLE_USERS
/* 392 */	{	RPL_USERSSTART,			":UserID   Terminal  Host" },
/* 393 */	{	RPL_USERS,				":%-8s %-9s %-8s" },
/* 394 */	{	RPL_ENDOFUSERS,			":End of Users" },
/* 395 */	{	RPL_NOUSERS,			":Nobody logged in." },
#else
/* 392 */	{	0,						(char *)NULL },
/* 393 */	{	0,						(char *)NULL },
/* 394 */	{	0,						(char *)NULL },
/* 395 */	{	0,						(char *)NULL },
#endif
/* 396 */	{	0,						(char *)NULL },
/* 397 */	{	0,						(char *)NULL },
/* 398 */	{	0,						(char *)NULL },
/* 399 */	{	0,						(char *)NULL },
/* 400 */	{	0,						(char *)NULL },
/* 401 */	{	ERR_NOSUCHNICK,			"%s :No such nick/channel" },
/* 402 */	{	ERR_NOSUCHSERVER,		"%s :No such server" },
/* 403 */	{	ERR_NOSUCHCHANNEL,		"%s :No such channel" },
/* 404 */	{	ERR_CANNOTSENDTOCHAN,	"%s :Cannot send to channel" },
/* 405 */	{	ERR_TOOMANYCHANNELS,
				"%s :You have joined too many channels" },
/* 406 */	{	ERR_WASNOSUCHNICK,		"%s :There was no such nickname" },
/* 407 */	{	ERR_TOOMANYTARGETS,
				"%s :Duplicate recipients. No message delivered" },
/* 408 */	{	ERR_NOSUCHSERVICE,		(char *)NULL },
/* 409 */	{	ERR_NOORIGIN,			":No origin specified" },
/* 410 */	{	0,						(char *)NULL },
/* 411 */	{	ERR_NORECIPIENT,		":No recipient given (%s)" },
/* 412 */	{	ERR_NOTEXTTOSEND,		":No text to send" },
/* 413 */	{	ERR_NOTOPLEVEL,			"%s :No toplevel domain specified" },
/* 414 */	{	ERR_WILDTOPLEVEL,		"%s :Wildcard in toplevel Domain" },
/* 415 */	{	0,						(char *)NULL },
/* 416 */	{	0,						(char *)NULL },
/* 417 */	{	0,						(char *)NULL },
/* 418 */	{	0,						(char *)NULL },
/* 419 */	{	0,						(char *)NULL },
/* 420 */	{	0,						(char *)NULL },
/* 421 */	{	ERR_UNKNOWNCOMMAND,		"%s :Unknown command" },
/* 422 */	{	ERR_NOMOTD,				":MOTD File is missing" },
/* 423 */	{	ERR_NOADMININFO,
				"%s :No administrative info available" },
/* 424 */	{	ERR_FILEERROR,			":File error doing %s on %s" },
/* 425 */	{	0,						(char *)NULL },
/* 426 */	{	0,						(char *)NULL },
/* 427 */	{	0,						(char *)NULL },
/* 428 */	{	0,						(char *)NULL },
/* 429 */	{	0,						(char *)NULL },
/* 430 */	{	0,						(char *)NULL },
/* 431 */	{	ERR_NONICKNAMEGIVEN,	":No nickname given" },
/* 432 */	{	ERR_ERRONEUSNICKNAME,	"%s :Erroneus Nickname" },
/* 433 */	{	ERR_NICKNAMEINUSE,		"%s :Nickname is already in use." },
/* 434 */	{	ERR_SERVICENAMEINUSE,	(char *)NULL },
/* 435 */	{	ERR_SERVICECONFUSED,	(char *)NULL },
/* 436 */	{	ERR_NICKCOLLISION,		"%s :Nickname collision KILL" },
/* 437 */	{	ERR_UNAVAILRESOURCE,	"%s :Nick/channel is temporarily unavailable" },
/* 438 */	{	0,						(char *)NULL },
/* 439 */	{	0,						(char *)NULL },
/* 440 */	{	0,						(char *)NULL },
/* 441 */	{	ERR_USERNOTINCHANNEL,	"%s %s :They aren't on that channel" },
/* 442 */	{	ERR_NOTONCHANNEL,		"%s :You're not on that channel" },
/* 443 */	{	ERR_USERONCHANNEL,		"%s %s :is already on channel" },
/* 444 */	{	ERR_NOLOGIN,			"%s :User not logged in" },
/* 445 */	{	ERR_SUMMONDISABLED,		":SUMMON has been disabled" },
/* 446 */	{	ERR_USERSDISABLED,		":USERS has been disabled" },
/* 447 */	{	0,						(char *)NULL },
/* 448 */	{	0,						(char *)NULL },
/* 449 */	{	0,						(char *)NULL },
/* 450 */	{	0,						(char *)NULL },
/* 451 */	{	ERR_NOTREGISTERED,		":You have not registered" },
/* 452 */	{	0,						(char *)NULL },
/* 453 */	{	0,						(char *)NULL },
/* 454 */	{	0,						(char *)NULL },
/* 455 */	{	0,						(char *)NULL },
/* 456 */	{	0,						(char *)NULL },
/* 457 */	{	0,						(char *)NULL },
/* 458 */	{	0,						(char *)NULL },
/* 459 */	{	0,						(char *)NULL },
/* 460 */	{	0,						(char *)NULL },
/* 461 */	{	ERR_NEEDMOREPARAMS,		"%s :Not enough parameters" },
/* 462 */	{	ERR_ALREADYREGISTRED,	":You may not reregister" },
/* 463 */	{	ERR_NOPERMFORHOST,
				":Your host isn't among the privileged" },
/* 464 */	{	ERR_PASSWDMISMATCH,		":Password Incorrect" },
/* 465 */	{	ERR_YOUREBANNEDCREEP,
				":You are banned from this server: %s" },
/* 466 */	{	ERR_YOUWILLBEBANNED,	(char *)NULL },
/* 467 */	{	ERR_KEYSET,				"%s :Channel key already set" },
/* 468 */	{	0,						(char *)NULL },
/* 469 */	{	0,						(char *)NULL },
/* 470 */	{	0,						(char *)NULL },
/* 471 */	{	ERR_CHANNELISFULL,		"%s :Cannot join channel (+l)" },
/* 472 */	{	ERR_UNKNOWNMODE,		"%c :is unknown mode char to me" },
/* 473 */	{	ERR_INVITEONLYCHAN,		"%s :Cannot join channel (+i)" },
/* 474 */	{	ERR_BANNEDFROMCHAN,		"%s :Cannot join channel (+b)" },
/* 475 */	{	ERR_BADCHANNELKEY,		"%s :Cannot join channel (+k)" },
/* 476 */	{	ERR_BADCHANMASK,		"%s :Bad Channel Mask" },
/* 477 */	{	0,						(char *)NULL },
/* 478 */	{	0,						(char *)NULL },
/* 479 */	{	0,						(char *)NULL },
/* 480 */	{	0,						(char *)NULL },
/* 481 */	{	ERR_NOPRIVILEGES,
				":Permission Denied- You're not an IRC operator" },
/* 482 */	{	ERR_CHANOPRIVSNEEDED,	"%s :You're not channel operator" },
/* 483 */	{	ERR_CANTKILLSERVER,		":You cant kill a server!" },
/* 484 */	{	0,						(char *)NULL },
/* 485 */	{	0,						(char *)NULL },
/* 486 */	{	0,						(char *)NULL },
/* 487 */	{	0,						(char *)NULL },
/* 488 */	{	0,						(char *)NULL },
/* 489 */	{	0,						(char *)NULL },
/* 490 */	{	0,						(char *)NULL },
/* 491 */	{	ERR_NOOPERHOST,			":No O-lines for your host" },
/* 492 */	{	ERR_NOSERVICEHOST,		(char *)NULL },
/* 493 */	{	0,						(char *)NULL },
/* 494 */	{	0,						(char *)NULL },
/* 495 */	{	0,						(char *)NULL },
/* 496 */	{	0,						(char *)NULL },
/* 497 */	{	0,						(char *)NULL },
/* 498 */	{	0,						(char *)NULL },
/* 499 */	{	0,						(char *)NULL },
/* 500 */	{	0,						(char *)NULL },
/* 501 */	{	ERR_UMODEUNKNOWNFLAG,	":Unknown MODE flag %c" },
/* 502 */	{	ERR_USERSDONTMATCH,		":Cant change mode for other users" },
/* 503 */	{	ERR_GHOSTEDCLIENT,
				":Message could not be delivered to %s" },
/* 504 */	{	0,						(char *)NULL }
};

static char **_ircd_replyarr;
static int _ircd_replyarrsize;

int numeric_init_reload(char **replyarr, int replyarrsize)
{
	_ircd_replyarr = replyarr;
	_ircd_replyarrsize = replyarrsize;
	return 0;
}

int numeric_deinit_reload(char ***replyarr, int *replyarrsize)
{
	if (replyarr)
		*replyarr = _ircd_replyarr;
	if (replyarrsize)
		*replyarrsize = _ircd_replyarrsize;
	return 0;
}

void numeric_init(void)
{
	IRCD_REPLY	*n;

	_ircd_replyarrsize = 200 + sizeof(_ircd_replies)/sizeof(IRCD_REPLY);
	_ircd_replyarr = (char **)MyMalloc(sizeof(char *)*_ircd_replyarrsize);
	for(n=_ircd_lreplies;(char *)n < ((char *)_ircd_lreplies)+sizeof(_ircd_lreplies);n++)
	{
		if (!n->num_val || !n->num_form)
			continue;
		assert(!_ircd_replyarr[n->num_val]);
		_ircd_replyarr[n->num_val] = (char *)MyMalloc(4+NICKLEN+1+strlen(n->num_form)+1);
		sprintf(_ircd_replyarr[n->num_val], "%03d ", n->num_val);
		*(_ircd_replyarr[n->num_val]+NICKLEN+4) = ' ';
		strcpy(_ircd_replyarr[n->num_val]+NICKLEN+5, n->num_form);
	}
	for(n=_ircd_replies;(char *)n < ((char *)_ircd_replies)+sizeof(_ircd_replies);n++)
	{
		if (!n->num_val || !n->num_form)
			continue;
		assert(!_ircd_replyarr[n->num_val]);
		_ircd_replyarr[n->num_val] = (char *)MyMalloc(4+NICKLEN+1+strlen(n->num_form)+1);
		sprintf(_ircd_replyarr[n->num_val], "%03d ", n->num_val);
		*(_ircd_replyarr[n->num_val]+NICKLEN+4) = ' ';
		strcpy(_ircd_replyarr[n->num_val]+NICKLEN+5, n->num_form);
	}
}

#if 0
/* this version has extra spaces in it...but...it's more efficient.
**   apparently vIRC breaks on this, though.  stupid client or stupid me?
*/
char *ircd_reply(int numeric, char *name)
{
	char	*str = _ircd_replyarr[numeric];
	char	*nptr = str + 4;
	int		len = NICKLEN;

	assert(str != NULL);
	if (!name)
		name = "*";
	for(;*name;len--)
	{
		if (!len)
		{
			return NULL; /* kludge--this will get caught in send_* functions */
		}
		*nptr++ = *name++;
	}
	while(len--)
		*nptr++ = ' ';
	return str;
}
#endif

/* no extra spaces in this version */
char *ircd_reply(int numeric, char *name)
{
	char			*str = _ircd_replyarr[numeric];
	int				len = NICKLEN;
	static char		tempbuf[512];
	char			*dptr = tempbuf;

	assert(str != NULL);
	if (!name)
		name = "*";
	*dptr++ = *str++;
	*dptr++ = *str++;
	*dptr++ = *str++;
	*dptr++ = *str++;
	for(;*name;len--)
	{
		if (!len)
		{
			return NULL; /* kludge--this will get caught in send_* functions */
		}
		*dptr++ = *name++;
	}
	str += NICKLEN;
	while((*dptr++ = *str++));
	return tempbuf;
}

int do_numeric(IRCD_CLIENT *cptr, IRCD_CLIENT *from, int numeric, char *prefix, char *args)
{
	char			*target = ircd_nextarg(&args, 0);
	char			*rest;
	IRCD_CLIENT		*tptr;
	int				colon;

	if (!args || !*args)
		return 2;
	colon = (*args == ':');
	rest = ircd_nextarg(&args, NEXTARG_FLAGS_LASTARG);
	if (!rest || !*rest)
		return 2;

	tptr = hash_find_client(target);
	if (!tptr)
		return 2;

	if (IsClient(from))
		send_prefix_to_client(from, tptr, "%d %s %s%s", numeric, target,
			colon ? ":" : "", rest);
	else
		send_to_client(tptr, ":%s %d %s %s%s", prefix, numeric, target,
			colon ? ":" : "", rest);

	return 2;
}
