/*
**   Malloc debugging routines (dmalloc.h)
**   Copyright (C) 1999 Chris Behrens
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __DMALLOC_H__
#define __DMALLOC_H__

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>


typedef struct _dmalloc_fptr    DMALLOC_FPTR;
typedef struct _dmalloc_pptr    DMALLOC_PPTR;

struct _dmalloc_fptr
{ 
	unsigned int	hashv;

    char            *file;
    int             line;
    int             nmallocs;
    DMALLOC_PPTR    *mallocs;
    DMALLOC_FPTR    *prev;
    DMALLOC_FPTR    *next;
};  
    
struct _dmalloc_pptr
{       
	int             size;
#define DM_PPTR_FLAGS_FREED	0x001
	int				flags;
	char            *ptr;
	DMALLOC_FPTR    *fptr;
   
	DMALLOC_PPTR    *f_next;
	DMALLOC_PPTR    *f_prev;
	DMALLOC_PPTR    *p_next;
	DMALLOC_PPTR    *p_prev;
};


DMALLOC_FPTR *dmalloc_file_find(char *file, int line);
int dmalloc_init(int phashsize, int fhashsize, int fbsize, int freecache);
void *dmalloc_malloc(int size, char *file, int line);
void *dmalloc_realloc(void *ptr, int size, char *file, int line);
char *dmalloc_strdup(char *string, char *file, int line);
void dmalloc_free(void *ptr, char *file, int line);
void dmalloc_file_get_start(void);
DMALLOC_FPTR *dmalloc_file_get_next(void);
int dmalloc_check(void);
int dmalloc_hash_info(void);
int dmalloc_pptr_exists(char *ptr, u_int ssize);

#endif
