/*
**   Malloc debugging routines (dmalloc_test.c)
**   Copyright (C) 1999 Chris Behrens
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include "dmalloc.h"

#if 1
#define MyMalloc(__x)	dmalloc_malloc((__x), __FILE__, __LINE__)
#define MyFree(__x)		dmalloc_free((__x), __FILE__, __LINE__)
#define MyStrdup(__x)	dmalloc_strdup((__x), __FILE__, __LINE__)
#define MyRealloc(__x, __y)	dmalloc_realloc((__x), (__y), __FILE__, __LINE__)
#else

#define MyMalloc malloc
#define MyFree free
#endif

int wuff()
{
	char *ptr = MyMalloc(1024);
	char *fptr = MyMalloc(123123);
	char *ptrs[1000];
	int i;

	fptr[0] = 'a';

	for(i=0;i<1000;i++)
		ptrs[i] = MyMalloc(i*2);

	dmalloc_hash_info();
	MyFree(ptr);


	for(i=0;i<1000;i+=2)
	{
		MyFree(ptrs[i]);
	}

	return 0;
}

int main()
{
	char	*ptr;
	DMALLOC_FPTR	*fptr;

	dmalloc_init(32*1024-1, 32*1024-1, 8, 1024*1024);

	ptr = MyMalloc(123123);
	*ptr = 1;

	MyFree(ptr);
	MyFree(ptr);

	wuff();

	printf("dumping\n");
	fflush(stdout);

	while((fptr = dmalloc_file_get_next()) != NULL)
		printf("%s:%i: %i mallocs\n", fptr->file, fptr->line, fptr->nmallocs);

	*(ptr-2) = 'w';

	dmalloc_check();

	MyFree(ptr);

	exit(0);
}

