/*
**   IRC - Internet Relay Chat, util/mkpasswd.c
**   Copyright (C) 1999 Chris Behrens
**
**   Portions Copyright (C) 1990 Jarkko Oikarinen and
**                      University of Oulu, Computing Center
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
** Originally written by Nelson Minar (minar@reed.edu)
**  -- copyright 1991, all rights reserved.
*/


#include "ircd_includes.h"

static char *_mkpasswd(char *string, char *saltstring);

static char *_mkpasswd(char *string, char *saltstring)
{
	static char saltChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
	char	salt[9];
	int		i;

	if (saltstring)
		strncpy(salt, saltstring, sizeof(salt)-1);
	else
	{
		for(i=0;i<8;i++)
			salt[i] = saltChars[(random() & 0x3f)];
	}
	salt[sizeof(salt)-1] = '\0';
	return crypt(string, salt);
}


int main(int argc, char **argv)
{
	char			*plaintext;
	struct timeval	tv;

	gettimeofday(&tv, NULL);
	srandom(tv.tv_sec/10000 * (tv.tv_usec+1));
	plaintext = getpass("plaintext: ");
	printf("%s\n", _mkpasswd(plaintext, argc < 2 ? NULL : argv[1]));
	return 0;
}

